@@ -25,7 +25,7 @@
 
 /**
  * ClutterOffscreenEffect:
- * 
+ *
  * Base class for effects using offscreen buffers
  *
  * #ClutterOffscreenEffect is an abstract class that can be used by
@@ -52,7 +52,7 @@
  * function, which encapsulates the effective painting of the texture that
  * contains the result of the offscreen redirection.
  *
- * The size of the target material is defined to be as big as the
+ * The size of the target pipeline is defined to be as big as the
  * transformed size of the [class@Actor] using the offscreen effect.
  * Sub-classes of #ClutterOffscreenEffect can change the texture creation
  * code to provide bigger textures by overriding the
@@ -85,7 +85,7 @@
  * of [vfunc@OffscreenEffect.create_pipeline].
  */
 
-#include "clutter/clutter-build-config.h"
+#include "config.h"
 
 #include "clutter/clutter-offscreen-effect.h"
 
@@ -103,11 +103,11 @@
 #include "clutter/clutter-paint-volume-private.h"
 #include "clutter/clutter-actor-box-private.h"
 
-struct _ClutterOffscreenEffectPrivate
+typedef struct _ClutterOffscreenEffectPrivate
 {
   CoglOffscreen *offscreen;
   CoglPipeline *pipeline;
-  CoglHandle texture;
+  CoglTexture *texture;
 
   ClutterActor *actor;
   ClutterActor *stage;
@@ -123,7 +123,7 @@ struct _ClutterOffscreenEffectPrivate
   int target_height;
 
   gulong purge_handler_id;
-};
+} ClutterOffscreenEffectPrivate;
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (ClutterOffscreenEffect,
                                      clutter_offscreen_effect,
@@ -134,7 +134,8 @@ clutter_offscreen_effect_set_actor (ClutterActorMeta *meta,
                                     ClutterActor     *actor)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (meta);
-  ClutterOffscreenEffectPrivate *priv = self->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
   ClutterActorMetaClass *meta_class;
 
   meta_class = CLUTTER_ACTOR_META_CLASS (clutter_offscreen_effect_parent_class);
@@ -147,24 +148,26 @@ clutter_offscreen_effect_set_actor (ClutterActorMeta *meta,
   priv->actor = clutter_actor_meta_get_actor (meta);
 }
 
-static CoglHandle
+static CoglTexture*
 clutter_offscreen_effect_real_create_texture (ClutterOffscreenEffect *effect,
+                                              CoglContext            *cogl_context,
                                               gfloat                  width,
                                               gfloat                  height)
 {
-  CoglContext *ctx =
-    clutter_backend_get_cogl_context (clutter_get_default_backend ());
-
-  return cogl_texture_2d_new_with_size (ctx, MAX (width, 1), MAX (height, 1));
+  return cogl_texture_2d_new_with_size (cogl_context,
+                                        (int) MAX (width, 1),
+                                        (int) MAX (height, 1));
 }
 
 static void
 ensure_pipeline_filter_for_scale (ClutterOffscreenEffect *self,
                                   float                   resource_scale)
 {
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
   CoglPipelineFilter filter;
 
-  if (!self->priv->pipeline)
+  if (!priv->pipeline)
     return;
 
   /* If no fractional scaling is set, we're always going to render the texture
@@ -178,7 +181,7 @@ ensure_pipeline_filter_for_scale (ClutterOffscreenEffect *self,
   else
     filter = COGL_PIPELINE_FILTER_LINEAR;
 
-  cogl_pipeline_set_layer_filters (self->priv->pipeline, 0 /* layer_index */,
+  cogl_pipeline_set_layer_filters (priv->pipeline, 0 /* layer_index */,
                                    filter, filter);
 }
 
@@ -186,9 +189,9 @@ static CoglPipeline *
 clutter_offscreen_effect_real_create_pipeline (ClutterOffscreenEffect *effect,
                                                CoglTexture            *texture)
 {
-  ClutterOffscreenEffectPrivate *priv = effect->priv;
-  CoglContext *ctx =
-    clutter_backend_get_cogl_context (clutter_get_default_backend ());
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (effect);
+  CoglContext *ctx = cogl_texture_get_context (texture);
   CoglPipeline *pipeline;
   float resource_scale;
 
@@ -204,7 +207,10 @@ clutter_offscreen_effect_real_create_pipeline (ClutterOffscreenEffect *effect,
 static void
 video_memory_purged (ClutterOffscreenEffect *self)
 {
-  g_clear_object (&self->priv->offscreen);
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
+
+  g_clear_object (&priv->offscreen);
 }
 
 static gboolean
@@ -216,9 +222,13 @@ update_fbo (ClutterEffect *effect,
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
   ClutterOffscreenEffectClass *offscreen_class =
     CLUTTER_OFFSCREEN_EFFECT_GET_CLASS (self);
-  ClutterOffscreenEffectPrivate *priv = self->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
   ClutterActor *stage_actor;
   CoglOffscreen *offscreen;
+  ClutterContext *context;
+  ClutterBackend *backend;
+  CoglContext *cogl_context;
   g_autoptr (GError) error = NULL;
 
   stage_actor = clutter_actor_get_stage (priv->actor);
@@ -251,16 +261,20 @@ update_fbo (ClutterEffect *effect,
   if (priv->target_width == target_width &&
       priv->target_height == target_height &&
       priv->offscreen != NULL)
-  {
-    ensure_pipeline_filter_for_scale (self, resource_scale);
-    return TRUE;
-  }
+    {
+      ensure_pipeline_filter_for_scale (self, resource_scale);
+      return TRUE;
+    }
 
-  g_clear_pointer (&priv->texture, cogl_object_unref);
+  g_clear_object (&priv->texture);
   g_clear_object (&priv->offscreen);
 
+  context = clutter_actor_get_context (priv->actor);
+  backend = clutter_context_get_backend (context);
+  cogl_context = clutter_backend_get_cogl_context (backend);
   priv->texture =
-    clutter_offscreen_effect_create_texture (self, target_width, target_height);
+    clutter_offscreen_effect_create_texture (self, cogl_context,
+                                             target_width, target_height);
   if (priv->texture == NULL)
     return FALSE;
 
@@ -274,7 +288,7 @@ update_fbo (ClutterEffect *effect,
                  error->message);
 
       g_object_unref (offscreen);
-      cogl_clear_object (&priv->pipeline);
+      g_clear_object (&priv->pipeline);
 
       priv->target_width = 0;
       priv->target_height = 0;
@@ -284,7 +298,7 @@ update_fbo (ClutterEffect *effect,
 
   priv->offscreen = offscreen;
 
-  cogl_clear_object (&priv->pipeline);
+  g_clear_object (&priv->pipeline);
   priv->pipeline = offscreen_class->create_pipeline (self, priv->texture);
 
   return TRUE;
@@ -296,7 +310,8 @@ clutter_offscreen_effect_pre_paint (ClutterEffect       *effect,
                                     ClutterPaintContext *paint_context)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
-  ClutterOffscreenEffectPrivate *priv = self->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
   CoglFramebuffer *offscreen;
   ClutterActorBox raw_box, box;
   ClutterActor *stage;
@@ -340,8 +355,8 @@ clutter_offscreen_effect_pre_paint (ClutterEffect       *effect,
       box = raw_box;
       _clutter_actor_box_enlarge_for_effects (&box);
 
-      priv->fbo_offset_x = box.x1;
-      priv->fbo_offset_y = box.y1;
+      priv->fbo_offset_x = (int) box.x1;
+      priv->fbo_offset_y = (int) box.y1;
     }
   else
     {
@@ -350,8 +365,8 @@ clutter_offscreen_effect_pre_paint (ClutterEffect       *effect,
       box = raw_box;
       _clutter_actor_box_enlarge_for_effects (&box);
 
-      priv->fbo_offset_x = box.x1 - raw_box.x1;
-      priv->fbo_offset_y = box.y1 - raw_box.y1;
+      priv->fbo_offset_x = (int) (box.x1 - raw_box.x1);
+      priv->fbo_offset_y = (int) (box.y1 - raw_box.y1);
     }
 
   clutter_actor_box_scale (&box, ceiled_resource_scale);
@@ -361,7 +376,7 @@ clutter_offscreen_effect_pre_paint (ClutterEffect       *effect,
   target_height = ceilf (target_height);
 
   /* First assert that the framebuffer is the right size... */
-  if (!update_fbo (effect, target_width, target_height, resource_scale))
+  if (!update_fbo (effect, (int) target_width, (int) target_height, resource_scale))
     goto disable_effect;
 
   offscreen = COGL_FRAMEBUFFER (priv->offscreen);
@@ -412,21 +427,22 @@ clutter_offscreen_effect_real_paint_target (ClutterOffscreenEffect *effect,
                                             ClutterPaintNode       *node,
                                             ClutterPaintContext    *paint_context)
 {
-  ClutterOffscreenEffectPrivate *priv = effect->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (effect);
   ClutterPaintNode *pipeline_node;
-  guint8 paint_opacity;
+  float paint_opacity;
+  CoglColor color;
 
-  paint_opacity = clutter_actor_get_paint_opacity (priv->actor);
+  paint_opacity = clutter_actor_get_paint_opacity (priv->actor) / 255.0f;
 
-  cogl_pipeline_set_color4ub (priv->pipeline,
-                              paint_opacity,
-                              paint_opacity,
-                              paint_opacity,
-                              paint_opacity);
+  cogl_color_init_from_4f (&color,
+                           paint_opacity, paint_opacity,
+                           paint_opacity, paint_opacity);
+  cogl_pipeline_set_color (priv->pipeline, &color);
 
   pipeline_node = clutter_pipeline_node_new (priv->pipeline);
   clutter_paint_node_set_static_name (pipeline_node,
-                                      "ClutterOffscreenEffect (pipeline)");
+                                      G_OBJECT_TYPE_NAME (effect));
   clutter_paint_node_add_child (node, pipeline_node);
 
   /* At this point we are in stage coordinates translated so if
@@ -449,16 +465,17 @@ clutter_offscreen_effect_paint_texture (ClutterOffscreenEffect *effect,
                                         ClutterPaintNode       *node,
                                         ClutterPaintContext    *paint_context)
 {
-  ClutterOffscreenEffectPrivate *priv = effect->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (effect);
   graphene_matrix_t transform;
   float unscale;
 
-  unscale = 1.0 / clutter_actor_get_resource_scale (priv->actor);
-  graphene_matrix_init_scale (&transform, unscale, unscale, 1.0);
+  unscale = 1.0f / clutter_actor_get_resource_scale (priv->actor);
+  graphene_matrix_init_scale (&transform, unscale, unscale, 1.0f);
   graphene_matrix_translate (&transform,
                              &GRAPHENE_POINT3D_INIT (priv->fbo_offset_x,
                                                      priv->fbo_offset_y,
-                                                     0.0));
+                                                     0.0f));
 
   if (!graphene_matrix_is_identity (&transform))
     {
@@ -473,7 +490,7 @@ clutter_offscreen_effect_paint_texture (ClutterOffscreenEffect *effect,
       node = transform_node;
     }
 
-  /* paint the target material; this is virtualized for
+  /* paint the target pipeline; this is virtualized for
    * sub-classes that require special hand-holding
    */
   clutter_offscreen_effect_paint_target (effect, node, paint_context);
@@ -485,7 +502,8 @@ clutter_offscreen_effect_post_paint (ClutterEffect       *effect,
                                      ClutterPaintContext *paint_context)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
-  ClutterOffscreenEffectPrivate *priv = self->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
 
   g_warn_if_fail (priv->offscreen);
   g_warn_if_fail (priv->pipeline);
@@ -499,7 +517,8 @@ add_actor_node (ClutterOffscreenEffect *offscreen_effect,
                 ClutterPaintNode       *node,
                 int                     paint_opacity)
 {
-  ClutterOffscreenEffectPrivate *priv = offscreen_effect->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (offscreen_effect);
   ClutterPaintNode *actor_node;
 
   actor_node = clutter_actor_node_new (priv->actor, paint_opacity);
@@ -514,7 +533,8 @@ clutter_offscreen_effect_paint_node (ClutterEffect           *effect,
                                      ClutterEffectPaintFlags  flags)
 {
   ClutterOffscreenEffect *offscreen_effect = CLUTTER_OFFSCREEN_EFFECT (effect);
-  ClutterOffscreenEffectPrivate *priv = offscreen_effect->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (offscreen_effect);
   ClutterPaintNode *layer_node;
   CoglFramebuffer *fb;
 
@@ -535,7 +555,8 @@ clutter_offscreen_effect_paint (ClutterEffect           *effect,
                                 ClutterEffectPaintFlags  flags)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
-  ClutterOffscreenEffectPrivate *priv = self->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
   ClutterEffectClass *parent_class =
     CLUTTER_EFFECT_CLASS (clutter_offscreen_effect_parent_class);
 
@@ -562,7 +583,8 @@ clutter_offscreen_effect_set_enabled (ClutterActorMeta *meta,
   ClutterActorMetaClass *parent_class =
     CLUTTER_ACTOR_META_CLASS (clutter_offscreen_effect_parent_class);
   ClutterOffscreenEffect *offscreen_effect = CLUTTER_OFFSCREEN_EFFECT (meta);
-  ClutterOffscreenEffectPrivate *priv = offscreen_effect->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (offscreen_effect);
 
   g_clear_object (&priv->offscreen);
 
@@ -573,11 +595,12 @@ static void
 clutter_offscreen_effect_finalize (GObject *gobject)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (gobject);
-  ClutterOffscreenEffectPrivate *priv = self->priv;
+  ClutterOffscreenEffectPrivate *priv =
+    clutter_offscreen_effect_get_instance_private (self);
 
   g_clear_object (&priv->offscreen);
-  g_clear_pointer (&priv->texture, cogl_object_unref);
-  g_clear_pointer (&priv->pipeline, cogl_object_unref);
+  g_clear_object (&priv->texture);
+  g_clear_object (&priv->pipeline);
 
   G_OBJECT_CLASS (clutter_offscreen_effect_parent_class)->finalize (gobject);
 }
@@ -607,7 +630,6 @@ clutter_offscreen_effect_class_init (ClutterOffscreenEffectClass *klass)
 static void
 clutter_offscreen_effect_init (ClutterOffscreenEffect *self)
 {
-  self->priv = clutter_offscreen_effect_get_instance_private (self);
 }
 
 /**
@@ -622,19 +644,22 @@ clutter_offscreen_effect_init (ClutterOffscreenEffect *self)
  * implementation should update any references to the texture after
  * chaining-up to the parent's pre_paint implementation. This can be
  * used instead of [method@OffscreenEffect.get_texture] when the
- * effect subclass wants to paint using its own material.
+ * effect subclass wants to paint using its own pipeline.
  *
- * Return value: (transfer none): a #CoglHandle or %NULL. The
+ * Return value: (transfer none): a #CoglTexture or %NULL. The
  *   returned texture is owned by Clutter and it should not be
  *   modified or freed
  */
-CoglHandle
+CoglTexture*
 clutter_offscreen_effect_get_texture (ClutterOffscreenEffect *effect)
 {
+  ClutterOffscreenEffectPrivate *priv;
+
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
                         NULL);
 
-  return effect->priv->texture;
+  priv = clutter_offscreen_effect_get_instance_private (effect);
+  return priv->texture;
 }
 
 /**
@@ -654,10 +679,13 @@ clutter_offscreen_effect_get_texture (ClutterOffscreenEffect *effect)
 CoglPipeline *
 clutter_offscreen_effect_get_pipeline (ClutterOffscreenEffect *effect)
 {
+  ClutterOffscreenEffectPrivate *priv;
+
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
                         NULL);
 
-  return effect->priv->pipeline;
+  priv = clutter_offscreen_effect_get_instance_private (effect);
+  return priv->pipeline;
 }
 
 /**
@@ -692,15 +720,18 @@ clutter_offscreen_effect_paint_target (ClutterOffscreenEffect *effect,
  *   %NULL. The returned handle has its reference
  *   count increased.
  */
-CoglHandle
+CoglTexture*
 clutter_offscreen_effect_create_texture (ClutterOffscreenEffect *effect,
+                                         CoglContext            *context,
                                          gfloat                  width,
                                          gfloat                  height)
 {
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
                         NULL);
+  g_return_val_if_fail (COGL_IS_CONTEXT (context), NULL);
 
   return CLUTTER_OFFSCREEN_EFFECT_GET_CLASS (effect)->create_texture (effect,
+                                                                      context,
                                                                       width,
                                                                       height);
 }
@@ -730,7 +761,7 @@ clutter_offscreen_effect_get_target_size (ClutterOffscreenEffect *effect,
 
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect), FALSE);
 
-  priv = effect->priv;
+  priv = clutter_offscreen_effect_get_instance_private (effect);
 
   if (priv->texture == NULL)
     return FALSE;
