@@ -223,6 +223,9 @@ ensure_eis_pointer (MetaInputCaptureSession *session)
   eis_device_resume (eis_pointer);
 
   session->eis_pointer = eis_pointer;
+
+  if (session->state == INPUT_CAPTURE_STATE_ACTIVATED)
+    eis_device_start_emulating (session->eis_pointer, session->activation_id);
 }
 
 static MetaAnonymousFile *
@@ -294,6 +297,9 @@ ensure_eis_keyboard (MetaInputCaptureSession *session)
   eis_device_resume (eis_keyboard);
 
   session->eis_keyboard = eis_keyboard;
+
+  if (session->state == INPUT_CAPTURE_STATE_ACTIVATED)
+    eis_device_start_emulating (session->eis_keyboard, session->activation_id);
 }
 
 static void
@@ -1021,7 +1027,7 @@ handle_release (MetaDBusInputCaptureSession *object,
   meta_input_capture_session_deactivate (session);
 
   if (g_variant_lookup (arg_options, "cursor_position", "(dd)", &x, &y))
-    clutter_seat_warp_pointer (seat, x, y);
+    clutter_seat_warp_pointer (seat, (int) x, (int) y);
 
   if (session->handle)
     release_remote_access_handle (session);
@@ -1453,7 +1459,6 @@ meta_input_capture_session_process_event (MetaInputCaptureSession *session,
         ClutterScrollFinishFlags finish_flags;
         const double factor = 10.0;
         bool stop_x = false, stop_y = false;
-        double dx, dy;
 
         if (!session->eis_pointer)
           return TRUE;
