@@ -30,11 +30,11 @@
 #include "core/boxes-private.h"
 
 #define SCALE_FACTORS_PER_INTEGER 4
-#define SCALE_FACTORS_STEPS (1.0 / (float) SCALE_FACTORS_PER_INTEGER)
+#define SCALE_FACTORS_STEPS (1.0f / (float) SCALE_FACTORS_PER_INTEGER)
 #define MINIMUM_SCALE_FACTOR 1.0f
 #define MAXIMUM_SCALE_FACTOR 4.0f
 #define MINIMUM_LOGICAL_AREA (800 * 480)
-#define MAXIMUM_REFRESH_RATE_DIFF 0.001
+#define MAXIMUM_REFRESH_RATE_DIFF 0.001f
 
 typedef struct _MetaMonitorMode
 {
@@ -66,17 +66,6 @@ typedef struct _MetaMonitorPrivate
 
   MetaLogicalMonitor *logical_monitor;
 
-  /*
-   * The primary or first output for this monitor, 0 if we can't figure out.
-   * It can be matched to a winsys_id of a MetaOutput.
-   *
-   * This is used as an opaque token on reconfiguration when switching from
-   * clone to extended, to decide on what output the windows should go next
-   * (it's an attempt to keep windows on the same monitor, and preferably on
-   * the primary one).
-   */
-  uint64_t winsys_id;
-
   char *display_name;
 } MetaMonitorPrivate;
 
@@ -381,6 +370,16 @@ meta_monitor_get_max_bpc (MetaMonitor  *monitor,
   return meta_output_get_max_bpc (output, max_bpc);
 }
 
+MetaOutputRGBRange
+meta_monitor_get_rgb_range (MetaMonitor *monitor)
+{
+  MetaOutput *output;
+
+  output = meta_monitor_get_main_output (monitor);
+
+  return meta_output_peek_rgb_range (output);
+}
+
 gboolean
 meta_monitor_is_laptop_panel (MetaMonitor *monitor)
 {
@@ -392,6 +391,7 @@ meta_monitor_is_laptop_panel (MetaMonitor *monitor)
     case META_CONNECTOR_TYPE_eDP:
     case META_CONNECTOR_TYPE_LVDS:
     case META_CONNECTOR_TYPE_DSI:
+    case META_CONNECTOR_TYPE_DPI:
       return TRUE;
     default:
       return FALSE;
@@ -413,20 +413,23 @@ meta_monitor_is_same_as (MetaMonitor *monitor,
 {
   const MetaMonitorSpec *spec = meta_monitor_get_spec (monitor);
   const MetaMonitorSpec *other_spec = meta_monitor_get_spec (other_monitor);
+  gboolean spec_is_unknown;
+  gboolean other_spec_is_unknown;
 
-  if ((g_strcmp0 (spec->vendor, "unknown") == 0 ||
-       g_strcmp0 (spec->product, "unknown") == 0 ||
-       g_strcmp0 (spec->serial, "unknown") == 0) &&
-      (g_strcmp0 (other_spec->vendor, "unknown") == 0 ||
-       g_strcmp0 (other_spec->product, "unknown") == 0 ||
-       g_strcmp0 (other_spec->serial, "unknown") == 0))
-    {
-      MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
-      MetaMonitorPrivate *other_priv =
-        meta_monitor_get_instance_private (other_monitor);
+  spec_is_unknown =
+    g_strcmp0 (spec->vendor, "unknown") == 0 ||
+    g_strcmp0 (spec->product, "unknown") == 0 ||
+    g_strcmp0 (spec->serial, "unknown") == 0;
+  other_spec_is_unknown =
+    g_strcmp0 (other_spec->vendor, "unknown") == 0 ||
+    g_strcmp0 (other_spec->product, "unknown") == 0 ||
+    g_strcmp0 (other_spec->serial, "unknown") == 0;
 
-      return priv->winsys_id == other_priv->winsys_id;
-    }
+  if (spec_is_unknown && other_spec_is_unknown)
+    return g_strcmp0 (spec->connector, other_spec->connector) == 0;
+
+  if (spec_is_unknown || other_spec_is_unknown)
+    return FALSE;
 
   if (g_strcmp0 (spec->vendor, other_spec->vendor) != 0)
     return FALSE;
@@ -470,7 +473,7 @@ meta_monitor_get_physical_dimensions (MetaMonitor *monitor,
   *height_mm = output_info->height_mm;
 }
 
-CoglSubpixelOrder
+MetaSubpixelOrder
 meta_monitor_get_subpixel_order (MetaMonitor *monitor)
 {
   const MetaOutputInfo *output_info =
@@ -542,18 +545,18 @@ meta_monitor_get_connector_type (MetaMonitor *monitor)
   return output_info->connector_type;
 }
 
-MetaMonitorTransform
+MtkMonitorTransform
 meta_monitor_logical_to_crtc_transform (MetaMonitor          *monitor,
-                                        MetaMonitorTransform  transform)
+                                        MtkMonitorTransform  transform)
 {
   MetaOutput *output = meta_monitor_get_main_output (monitor);
 
   return meta_output_logical_to_crtc_transform (output, transform);
 }
 
-MetaMonitorTransform
-meta_monitor_crtc_to_logical_transform (MetaMonitor          *monitor,
-                                        MetaMonitorTransform  transform)
+MtkMonitorTransform
+meta_monitor_crtc_to_logical_transform (MetaMonitor         *monitor,
+                                        MtkMonitorTransform  transform)
 {
   MetaOutput *output = meta_monitor_get_main_output (monitor);
 
@@ -612,16 +615,21 @@ generate_mode_id (MetaMonitorModeSpec *monitor_mode_spec)
 {
   gboolean is_interlaced;
   char rate_str[G_ASCII_DTOSTR_BUF_SIZE];
+  gboolean is_vrr;
 
   is_interlaced = !!(monitor_mode_spec->flags & META_CRTC_MODE_FLAG_INTERLACE);
   g_ascii_formatd (rate_str, sizeof (rate_str),
                    "%.3f", monitor_mode_spec->refresh_rate);
 
-  return g_strdup_printf ("%dx%d%s@%s",
+  is_vrr = monitor_mode_spec->refresh_rate_mode ==
+           META_CRTC_REFRESH_RATE_MODE_VARIABLE;
+
+  return g_strdup_printf ("%dx%d%s@%s%s",
                           monitor_mode_spec->width,
                           monitor_mode_spec->height,
                           is_interlaced ? "i" : "",
-                          rate_str);
+                          rate_str,
+                          is_vrr ? "+vrr" : "");
 }
 
 static gboolean
@@ -657,7 +665,7 @@ meta_monitor_create_spec (MetaMonitor  *monitor,
   const MetaCrtcModeInfo *crtc_mode_info =
     meta_crtc_mode_get_info (crtc_mode);
 
-  if (meta_monitor_transform_is_rotated (output_info->panel_orientation_transform))
+  if (mtk_monitor_transform_is_rotated (output_info->panel_orientation_transform))
     {
       int temp = width;
       width = height;
@@ -668,6 +676,7 @@ meta_monitor_create_spec (MetaMonitor  *monitor,
     .width = width,
     .height = height,
     .refresh_rate = crtc_mode_info->refresh_rate,
+    .refresh_rate_mode = crtc_mode_info->refresh_rate_mode,
     .flags = crtc_mode_info->flags & HANDLED_CRTC_MODE_FLAGS
   };
 }
@@ -823,7 +832,6 @@ meta_monitor_normal_new (MetaMonitorManager *monitor_manager,
   monitor_priv->outputs = g_list_append (NULL, g_object_ref (output));
   meta_output_set_monitor (output, monitor);
 
-  monitor_priv->winsys_id = meta_output_get_id (output);
   meta_monitor_generate_spec (monitor);
 
   meta_monitor_normal_generate_modes (monitor_normal);
@@ -885,12 +893,12 @@ meta_monitor_normal_get_suggested_position (MetaMonitor *monitor,
 }
 
 static void
-meta_monitor_normal_calculate_crtc_pos (MetaMonitor          *monitor,
-                                        MetaMonitorMode      *monitor_mode,
-                                        MetaOutput           *output,
-                                        MetaMonitorTransform  crtc_transform,
-                                        int                  *out_x,
-                                        int                  *out_y)
+meta_monitor_normal_calculate_crtc_pos (MetaMonitor         *monitor,
+                                        MetaMonitorMode     *monitor_mode,
+                                        MetaOutput          *output,
+                                        MtkMonitorTransform  crtc_transform,
+                                        int                 *out_x,
+                                        int                 *out_y)
 {
   *out_x = 0;
   *out_y = 0;
@@ -960,7 +968,7 @@ add_tiled_monitor_outputs (MetaGpu          *gpu,
 static void
 calculate_tile_coordinate (MetaMonitor         *monitor,
                            MetaOutput          *output,
-                           MetaMonitorTransform crtc_transform,
+                           MtkMonitorTransform  crtc_transform,
                            int                 *out_x,
                            int                 *out_y)
 {
@@ -977,8 +985,8 @@ calculate_tile_coordinate (MetaMonitor         *monitor,
 
       switch (crtc_transform)
         {
-        case META_MONITOR_TRANSFORM_NORMAL:
-        case META_MONITOR_TRANSFORM_FLIPPED:
+        case MTK_MONITOR_TRANSFORM_NORMAL:
+        case MTK_MONITOR_TRANSFORM_FLIPPED:
           if ((other_output_info->tile_info.loc_v_tile ==
                output_info->tile_info.loc_v_tile) &&
               (other_output_info->tile_info.loc_h_tile <
@@ -990,8 +998,8 @@ calculate_tile_coordinate (MetaMonitor         *monitor,
                output_info->tile_info.loc_v_tile))
             y += other_output_info->tile_info.tile_h;
           break;
-        case META_MONITOR_TRANSFORM_180:
-        case META_MONITOR_TRANSFORM_FLIPPED_180:
+        case MTK_MONITOR_TRANSFORM_180:
+        case MTK_MONITOR_TRANSFORM_FLIPPED_180:
           if ((other_output_info->tile_info.loc_v_tile ==
                output_info->tile_info.loc_v_tile) &&
               (other_output_info->tile_info.loc_h_tile >
@@ -1003,8 +1011,8 @@ calculate_tile_coordinate (MetaMonitor         *monitor,
                output_info->tile_info.loc_v_tile))
             y += other_output_info->tile_info.tile_h;
           break;
-        case META_MONITOR_TRANSFORM_270:
-        case META_MONITOR_TRANSFORM_FLIPPED_270:
+        case MTK_MONITOR_TRANSFORM_270:
+        case MTK_MONITOR_TRANSFORM_FLIPPED_270:
           if ((other_output_info->tile_info.loc_v_tile ==
                output_info->tile_info.loc_v_tile) &&
               (other_output_info->tile_info.loc_h_tile >
@@ -1016,8 +1024,8 @@ calculate_tile_coordinate (MetaMonitor         *monitor,
                output_info->tile_info.loc_v_tile))
             x += other_output_info->tile_info.tile_h;
           break;
-        case META_MONITOR_TRANSFORM_90:
-        case META_MONITOR_TRANSFORM_FLIPPED_90:
+        case MTK_MONITOR_TRANSFORM_90:
+        case MTK_MONITOR_TRANSFORM_FLIPPED_90:
           if ((other_output_info->tile_info.loc_v_tile ==
                output_info->tile_info.loc_v_tile) &&
               (other_output_info->tile_info.loc_h_tile <
@@ -1131,6 +1139,9 @@ find_tiled_crtc_mode (MetaOutput   *output,
       if (crtc_mode_info->refresh_rate != reference_crtc_mode_info->refresh_rate)
         continue;
 
+      if (crtc_mode_info->refresh_rate_mode != reference_crtc_mode_info->refresh_rate_mode)
+        continue;
+
       if (crtc_mode_info->flags != reference_crtc_mode_info->flags)
         continue;
 
@@ -1245,9 +1256,24 @@ generate_tiled_monitor_modes (MetaMonitorTiled *monitor_tiled)
 
       if (!monitor_priv->preferred_mode)
         {
-          if (!best_mode ||
-              mode->spec.refresh_rate > best_mode->spec.refresh_rate)
-            best_mode = mode;
+          if (!best_mode)
+            {
+              best_mode = mode;
+              continue;
+            }
+
+          if (mode->spec.refresh_rate > best_mode->spec.refresh_rate)
+            {
+              best_mode = mode;
+              continue;
+            }
+
+          if (mode->spec.refresh_rate == best_mode->spec.refresh_rate &&
+              mode->spec.refresh_rate_mode > best_mode->spec.refresh_rate_mode)
+            {
+              best_mode = mode;
+              continue;
+            }
         }
     }
 
@@ -1433,6 +1459,13 @@ find_best_mode (MetaMonitor *monitor)
           best_mode = mode;
           continue;
         }
+
+      if (mode->spec.refresh_rate == best_mode->spec.refresh_rate &&
+          mode->spec.refresh_rate_mode > best_mode->spec.refresh_rate_mode)
+        {
+          best_mode = mode;
+          continue;
+        }
     }
 
   return best_mode;
@@ -1511,7 +1544,6 @@ meta_monitor_tiled_new (MetaMonitorManager *monitor_manager,
   monitor_priv->backend = meta_monitor_manager_get_backend (monitor_manager);
 
   monitor_tiled->tile_group_id = output_info->tile_info.group_id;
-  monitor_priv->winsys_id = meta_output_get_id (output);
 
   monitor_tiled->origin_output = output;
   add_tiled_monitor_outputs (meta_output_get_gpu (output), monitor_tiled);
@@ -1575,10 +1607,10 @@ meta_monitor_tiled_derive_layout (MetaMonitor  *monitor,
     }
 
   *layout = (MtkRectangle) {
-    .x = roundf (min_x),
-    .y = roundf (min_y),
-    .width = roundf (max_x - min_x),
-    .height = roundf (max_y - min_y)
+    .x = (int) roundf (min_x),
+    .y = (int) roundf (min_y),
+    .width = (int) roundf (max_x - min_x),
+    .height = (int) roundf (max_y - min_y)
   };
 }
 
@@ -1591,12 +1623,12 @@ meta_monitor_tiled_get_suggested_position (MetaMonitor *monitor,
 }
 
 static void
-meta_monitor_tiled_calculate_crtc_pos (MetaMonitor          *monitor,
-                                       MetaMonitorMode      *monitor_mode,
-                                       MetaOutput           *output,
-                                       MetaMonitorTransform  crtc_transform,
-                                       int                  *out_x,
-                                       int                  *out_y)
+meta_monitor_tiled_calculate_crtc_pos (MetaMonitor         *monitor,
+                                       MetaMonitorMode     *monitor_mode,
+                                       MetaOutput          *output,
+                                       MtkMonitorTransform  crtc_transform,
+                                       int                 *out_x,
+                                       int                 *out_y)
 {
   MetaMonitorModeTiled *mode_tiled = (MetaMonitorModeTiled *) monitor_mode;
 
@@ -1679,12 +1711,12 @@ gboolean
 meta_monitor_mode_spec_has_similar_size (MetaMonitorModeSpec *monitor_mode_spec,
                                          MetaMonitorModeSpec *other_monitor_mode_spec)
 {
-  const float target_ratio = 1.0;
+  const float target_ratio = 1.0f;
   /* The a size difference of 15% means e.g. 4K modes matches other 4K modes,
    * FHD (2K) modes other FHD modes, and HD modes other HD modes, but not each
    * other.
    */
-  const float epsilon = 0.15;
+  const float epsilon = 0.15f;
 
   return G_APPROX_VALUE (((float) monitor_mode_spec->width /
                           other_monitor_mode_spec->width) *
@@ -1701,6 +1733,8 @@ meta_monitor_mode_spec_equals (MetaMonitorModeSpec *monitor_mode_spec,
           monitor_mode_spec->height == other_monitor_mode_spec->height &&
           ABS (monitor_mode_spec->refresh_rate -
                other_monitor_mode_spec->refresh_rate) < MAXIMUM_REFRESH_RATE_DIFF &&
+          monitor_mode_spec->refresh_rate_mode ==
+          other_monitor_mode_spec->refresh_rate_mode &&
           monitor_mode_spec->flags == other_monitor_mode_spec->flags);
 }
 
@@ -1793,12 +1827,12 @@ meta_monitor_get_modes (MetaMonitor *monitor)
 }
 
 void
-meta_monitor_calculate_crtc_pos (MetaMonitor          *monitor,
-                                 MetaMonitorMode      *monitor_mode,
-                                 MetaOutput           *output,
-                                 MetaMonitorTransform  crtc_transform,
-                                 int                  *out_x,
-                                 int                  *out_y)
+meta_monitor_calculate_crtc_pos (MetaMonitor         *monitor,
+                                 MetaMonitorMode     *monitor_mode,
+                                 MetaOutput          *output,
+                                 MtkMonitorTransform  crtc_transform,
+                                 int                 *out_x,
+                                 int                 *out_y)
 {
   META_MONITOR_GET_CLASS (monitor)->calculate_crtc_pos (monitor,
                                                         monitor_mode,
@@ -1843,7 +1877,7 @@ calculate_scale (MetaMonitor                *monitor,
   int n_scales;
   float best_scale, best_dpi;
   int target_dpi;
-  const float scale_epsilon = 0.2;
+  const float scale_epsilon = 0.2f;
 
   /*
    * Somebody encoded the aspect ratio (16/9 or 16/10) instead of the physical
@@ -1856,7 +1890,7 @@ calculate_scale (MetaMonitor                *monitor,
   meta_monitor_get_physical_dimensions (monitor, &width_mm, &height_mm);
   if (width_mm == 0 || height_mm == 0)
     return 1.0;
-  diag_inches = sqrtf (width_mm * width_mm + height_mm * height_mm) / 25.4;
+  diag_inches = sqrtf (width_mm * width_mm + height_mm * height_mm) / 25.4f;
 
   /* Pick the appropriate target DPI based on screen size */
   if (diag_inches < UI_SCALE_LARGE_MIN_SIZE_INCHES)
@@ -1903,7 +1937,7 @@ calculate_scale (MetaMonitor                *monitor,
   if (constraints & META_MONITOR_SCALES_CONSTRAINT_NO_FRAC)
     {
       best_scale = floorf (MIN (scales[n_scales - 1],
-                                best_scale + 0.25 + scale_epsilon));
+                                best_scale + 0.25f + scale_epsilon));
     }
 
   return best_scale;
@@ -1940,8 +1974,8 @@ is_scale_valid_for_size (float width,
   if (scale < MINIMUM_SCALE_FACTOR || scale > MAXIMUM_SCALE_FACTOR)
     return FALSE;
 
-  return is_logical_size_large_enough (floorf (width / scale),
-                                       floorf (height / scale));
+  return is_logical_size_large_enough ((int) floorf (width / scale),
+                                       (int) floorf (height / scale));
 }
 
 gboolean
@@ -1960,11 +1994,11 @@ meta_monitor_mode_should_be_advertised (MetaMonitorMode *monitor_mode)
                                        monitor_mode->spec.height);
 }
 
-static float
-get_closest_scale_factor_for_resolution (float width,
-                                         float height,
-                                         float scale,
-                                         float threshold)
+float
+meta_get_closest_monitor_scale_factor_for_resolution (float width,
+                                                      float height,
+                                                      float scale,
+                                                      float threshold)
 {
   unsigned int i, j;
   float scaled_h;
@@ -1975,15 +2009,12 @@ get_closest_scale_factor_for_resolution (float width,
 
   best_scale = 0;
 
-  if (!is_scale_valid_for_size (width, height, scale))
-    return best_scale;
-
   if (fmodf (width, scale) == 0.0 && fmodf (height, scale) == 0.0)
     return scale;
 
   i = 0;
   found_one = FALSE;
-  base_scaled_w = floorf (width / scale);
+  base_scaled_w = (int) floorf (width / scale);
 
   do
     {
@@ -2061,9 +2092,13 @@ meta_monitor_calculate_supported_scales (MetaMonitor                 *monitor,
               float scale;
               float scale_value = i + j * SCALE_FACTORS_STEPS;
 
-              scale = get_closest_scale_factor_for_resolution (width, height,
-                                                               scale_value,
-                                                               max_bound);
+              if (!is_scale_valid_for_size (width, height, scale_value))
+                continue;
+
+              scale = meta_get_closest_monitor_scale_factor_for_resolution (width,
+                                                                            height,
+                                                                            scale_value,
+                                                                            max_bound);
               if (scale > 0.0)
                 g_array_append_val (supported_scales, scale);
             }
@@ -2109,6 +2144,12 @@ meta_monitor_mode_get_refresh_rate (MetaMonitorMode *monitor_mode)
   return monitor_mode->spec.refresh_rate;
 }
 
+MetaCrtcRefreshRateMode
+meta_monitor_mode_get_refresh_rate_mode (MetaMonitorMode *monitor_mode)
+{
+  return monitor_mode->spec.refresh_rate_mode;
+}
+
 MetaCrtcModeFlag
 meta_monitor_mode_get_flags (MetaMonitorMode *monitor_mode)
 {
@@ -2225,6 +2266,25 @@ meta_monitor_set_privacy_screen_enabled (MetaMonitor  *monitor,
   return meta_output_set_privacy_screen_enabled (output, enabled, error);
 }
 
+gboolean
+meta_monitor_get_min_refresh_rate (MetaMonitor *monitor,
+                                   int         *min_refresh_rate)
+{
+  const MetaOutputInfo *output_info =
+    meta_monitor_get_main_output_info (monitor);
+
+  return meta_output_info_get_min_refresh_rate (output_info,
+                                                min_refresh_rate);
+}
+
+MetaOutputColorspace
+meta_monitor_get_color_space (MetaMonitor *monitor)
+{
+  MetaOutput *output = meta_monitor_get_main_output (monitor);
+
+  return meta_output_peek_color_space (output);
+}
+
 gboolean
 meta_monitor_set_color_space (MetaMonitor           *monitor,
                               MetaOutputColorspace   color_space,
@@ -2236,8 +2296,9 @@ meta_monitor_set_color_space (MetaMonitor           *monitor,
   for (l = priv->outputs; l; l = l->next)
     {
       MetaOutput *output = l->data;
+      const MetaOutputInfo *output_info = meta_output_get_info (output);
 
-      if (!meta_output_is_color_space_supported (output, color_space))
+      if (!(output_info->supported_color_spaces & (1 << color_space)))
         {
           g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                                "The color space is not supported by this monitor");
@@ -2255,6 +2316,14 @@ meta_monitor_set_color_space (MetaMonitor           *monitor,
   return TRUE;
 }
 
+MetaOutputHdrMetadata *
+meta_monitor_get_hdr_metadata (MetaMonitor *monitor)
+{
+  MetaOutput *output = meta_monitor_get_main_output (monitor);
+
+  return meta_output_peek_hdr_metadata (output);
+}
+
 gboolean
 meta_monitor_set_hdr_metadata (MetaMonitor            *monitor,
                                MetaOutputHdrMetadata  *metadata,
@@ -2266,8 +2335,9 @@ meta_monitor_set_hdr_metadata (MetaMonitor            *monitor,
   for (l = priv->outputs; l; l = l->next)
     {
       MetaOutput *output = l->data;
+      const MetaOutputInfo *output_info = meta_output_get_info (output);
 
-      if (!meta_output_is_hdr_metadata_supported (output, metadata->eotf))
+      if (!(output_info->supported_hdr_eotfs & (1 << metadata->eotf)))
         {
           g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                                "HDR metadata is not supported by this monitor");
@@ -2318,8 +2388,8 @@ meta_parse_monitor_mode (const char *string,
     return FALSE;
   ptr++;
 
-  refresh_rate = g_ascii_strtod (ptr, &ptr);
-  if (refresh_rate == 0.0)
+  refresh_rate = (float) g_ascii_strtod (ptr, &ptr);
+  if (G_APPROX_VALUE (refresh_rate, 0.0f, FLT_EPSILON))
     return FALSE;
 
   if (ptr[0] != '\0')
@@ -2331,3 +2401,61 @@ out:
   *out_refresh_rate = refresh_rate;
   return TRUE;
 }
+
+gboolean
+meta_monitor_get_backlight_info (MetaMonitor *monitor,
+                                 int         *backlight_min,
+                                 int         *backlight_max)
+{
+  MetaOutput *main_output;
+  int value;
+
+  main_output = meta_monitor_get_main_output (monitor);
+  value = meta_output_get_backlight (main_output);
+  if (value >= 0)
+    {
+      const MetaOutputInfo *output_info = meta_output_get_info (main_output);
+      if (backlight_min)
+        *backlight_min = output_info->backlight_min;
+      if (backlight_max)
+        *backlight_max = output_info->backlight_max;
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
+
+void
+meta_monitor_set_backlight (MetaMonitor *monitor,
+                            int          value)
+{
+  MetaMonitorPrivate *priv = meta_monitor_get_instance_private (monitor);
+  GList *l;
+
+  for (l = priv->outputs; l; l = l->next)
+    {
+      MetaOutput *output = l->data;
+
+      meta_output_set_backlight (output, value);
+    }
+}
+
+gboolean
+meta_monitor_get_backlight (MetaMonitor *monitor,
+                            int         *value)
+{
+  if (meta_monitor_get_backlight_info (monitor, NULL, NULL))
+    {
+      MetaOutput *main_output;
+
+      main_output = meta_monitor_get_main_output (monitor);
+      *value = meta_output_get_backlight (main_output);
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
