@@ -21,8 +21,6 @@
 #pragma once
 
 #include <glib-object.h>
-#include <spa/param/video/format-utils.h>
-#include <spa/buffer/meta.h>
 
 #include "backends/meta-backend-private.h"
 #include "backends/meta-cursor-renderer.h"
@@ -47,6 +45,17 @@ typedef enum _MetaScreenCastRecordResult
   META_SCREEN_CAST_RECORD_RESULT_RECORDED_CURSOR = 1 << 1,
 } MetaScreenCastRecordResult;
 
+typedef enum _MetaScreenCastPaintPhase
+{
+  META_SCREEN_CAST_PAINT_PHASE_DETACHED,
+  META_SCREEN_CAST_PAINT_PHASE_PRE_PAINT,
+  META_SCREEN_CAST_PAINT_PHASE_PRE_SWAP_BUFFER,
+} MetaScreenCastPaintPhase;
+
+/* Declare some SPA types to avoid including the headers in too many places. */
+struct spa_meta_cursor;
+struct spa_video_info_raw;
+
 #define META_TYPE_SCREEN_CAST_STREAM_SRC (meta_screen_cast_stream_src_get_type ())
 G_DECLARE_DERIVABLE_TYPE (MetaScreenCastStreamSrc,
                           meta_screen_cast_stream_src,
@@ -64,18 +73,21 @@ struct _MetaScreenCastStreamSrcClass
   void (* enable) (MetaScreenCastStreamSrc *src);
   void (* disable) (MetaScreenCastStreamSrc *src);
   gboolean (* record_to_buffer) (MetaScreenCastStreamSrc  *src,
+                                 MetaScreenCastPaintPhase  paint_phase,
                                  int                       width,
                                  int                       height,
                                  int                       stride,
                                  uint8_t                  *data,
                                  GError                  **error);
-  gboolean (* record_to_framebuffer) (MetaScreenCastStreamSrc  *src,
-                                      CoglFramebuffer          *framebuffer,
-                                      GError                  **error);
+  gboolean (* record_to_framebuffer) (MetaScreenCastStreamSrc   *src,
+                                      MetaScreenCastPaintPhase   paint_phase,
+                                      CoglFramebuffer           *framebuffer,
+                                      GError                   **error);
   void (* record_follow_up) (MetaScreenCastStreamSrc *src);
 
   gboolean (* get_videocrop) (MetaScreenCastStreamSrc *src,
                               MtkRectangle            *crop_rect);
+  gboolean (* is_cursor_metadata_valid) (MetaScreenCastStreamSrc *src);
   void (* set_cursor_metadata) (MetaScreenCastStreamSrc *src,
                                 struct spa_meta_cursor  *spa_meta_cursor);
 
@@ -91,11 +103,13 @@ gboolean meta_screen_cast_stream_src_is_enabled (MetaScreenCastStreamSrc *src);
 
 MetaScreenCastRecordResult meta_screen_cast_stream_src_maybe_record_frame (MetaScreenCastStreamSrc  *src,
                                                                            MetaScreenCastRecordFlag  flags,
-                                                                           const cairo_region_t     *redraw_clip);
+                                                                           MetaScreenCastPaintPhase  paint_phase,
+                                                                           const MtkRegion          *redraw_clip);
 
 MetaScreenCastRecordResult meta_screen_cast_stream_src_maybe_record_frame_with_timestamp (MetaScreenCastStreamSrc  *src,
                                                                                           MetaScreenCastRecordFlag  flags,
-                                                                                          const cairo_region_t     *redraw_clip,
+                                                                                          MetaScreenCastPaintPhase  paint_phase,
+                                                                                          const MtkRegion          *redraw_clip,
                                                                                           int64_t                   frame_timestamp_us);
 
 gboolean meta_screen_cast_stream_src_pending_follow_up_frame (MetaScreenCastStreamSrc *src);
@@ -105,7 +119,7 @@ MetaScreenCastStream * meta_screen_cast_stream_src_get_stream (MetaScreenCastStr
 gboolean meta_screen_cast_stream_src_draw_cursor_into (MetaScreenCastStreamSrc  *src,
                                                        CoglTexture              *cursor_texture,
                                                        float                     scale,
-                                                       MetaMonitorTransform      transform,
+                                                       MtkMonitorTransform       transform,
                                                        uint8_t                  *data,
                                                        GError                  **error);
 
@@ -128,7 +142,7 @@ void meta_screen_cast_stream_src_set_cursor_sprite_metadata (MetaScreenCastStrea
                                                              int                      x,
                                                              int                      y,
                                                              float                    scale,
-                                                             MetaMonitorTransform     transform);
+                                                             MtkMonitorTransform      transform);
 
 gboolean meta_screen_cast_stream_src_uses_dma_bufs (MetaScreenCastStreamSrc *src);
 
