@@ -44,7 +44,6 @@ struct _MetaScreenCastVirtualStreamSrc
   MetaVirtualMonitor *virtual_monitor;
 
   gboolean cursor_bitmap_invalid;
-  gboolean hw_cursor_inhibited;
 
   struct {
     gboolean set;
@@ -61,14 +60,9 @@ struct _MetaScreenCastVirtualStreamSrc
   gulong monitors_changed_handler_id;
 };
 
-static void
-hw_cursor_inhibitor_iface_init (MetaHwCursorInhibitorInterface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (MetaScreenCastVirtualStreamSrc,
-                         meta_screen_cast_virtual_stream_src,
-                         META_TYPE_SCREEN_CAST_STREAM_SRC,
-                         G_IMPLEMENT_INTERFACE (META_TYPE_HW_CURSOR_INHIBITOR,
-                                                hw_cursor_inhibitor_iface_init))
+G_DEFINE_FINAL_TYPE (MetaScreenCastVirtualStreamSrc,
+                     meta_screen_cast_virtual_stream_src,
+                     META_TYPE_SCREEN_CAST_STREAM_SRC)
 
 static gboolean
 meta_screen_cast_virtual_stream_src_get_specs (MetaScreenCastStreamSrc *src,
@@ -100,7 +94,7 @@ view_from_src (MetaScreenCastStreamSrc *src)
   MetaRenderer *renderer = meta_backend_get_renderer (backend_from_src (src));
   MetaRendererView *view = meta_renderer_get_view_for_crtc (renderer, crtc);
 
-  return CLUTTER_STAGE_VIEW (view);
+  return view ? CLUTTER_STAGE_VIEW (view) : NULL;
 }
 
 static ClutterStage *
@@ -181,37 +175,7 @@ on_prepare_frame (ClutterStage                   *stage,
 }
 
 static void
-inhibit_hw_cursor (MetaScreenCastVirtualStreamSrc *virtual_src)
-{
-  MetaHwCursorInhibitor *inhibitor;
-  MetaBackend *backend;
-
-  g_return_if_fail (!virtual_src->hw_cursor_inhibited);
-
-  backend = backend_from_src (META_SCREEN_CAST_STREAM_SRC (virtual_src));
-  inhibitor = META_HW_CURSOR_INHIBITOR (virtual_src);
-  meta_backend_add_hw_cursor_inhibitor (backend, inhibitor);
-
-  virtual_src->hw_cursor_inhibited = TRUE;
-}
-
-static void
-uninhibit_hw_cursor (MetaScreenCastVirtualStreamSrc *virtual_src)
-{
-  MetaHwCursorInhibitor *inhibitor;
-  MetaBackend *backend;
-
-  g_return_if_fail (virtual_src->hw_cursor_inhibited);
-
-  backend = backend_from_src (META_SCREEN_CAST_STREAM_SRC (virtual_src));
-  inhibitor = META_HW_CURSOR_INHIBITOR (virtual_src);
-  meta_backend_remove_hw_cursor_inhibitor (backend, inhibitor);
-
-  virtual_src->hw_cursor_inhibited = FALSE;
-}
-
-static void
-actors_painted (MetaStage        *stage,
+on_after_paint (MetaStage        *stage,
                 ClutterStageView *view,
                 const MtkRegion  *redraw_clip,
                 ClutterFrame     *frame,
@@ -229,7 +193,8 @@ actors_painted (MetaStage        *stage,
 }
 
 static void
-add_watch (MetaScreenCastVirtualStreamSrc *virtual_src)
+setup_view (MetaScreenCastVirtualStreamSrc *virtual_src,
+            ClutterStageView               *view)
 {
   MetaScreenCastStreamSrc *src = META_SCREEN_CAST_STREAM_SRC (virtual_src);
   MetaScreenCastStream *stream = meta_screen_cast_stream_src_get_stream (src);
@@ -241,20 +206,18 @@ add_watch (MetaScreenCastVirtualStreamSrc *virtual_src)
     {
     case META_SCREEN_CAST_CURSOR_MODE_METADATA:
     case META_SCREEN_CAST_CURSOR_MODE_HIDDEN:
-      virtual_src->watch = meta_stage_watch_view (meta_stage,
-                                                  view_from_src (src),
-                                                  META_STAGE_WATCH_AFTER_ACTOR_PAINT,
-                                                  actors_painted,
-                                                  virtual_src);
+      meta_stage_view_inhibit_cursor_overlay (META_STAGE_VIEW (view));
       break;
     case META_SCREEN_CAST_CURSOR_MODE_EMBEDDED:
-      virtual_src->watch = meta_stage_watch_view (meta_stage,
-                                                  view_from_src (src),
-                                                  META_STAGE_WATCH_AFTER_PAINT,
-                                                  actors_painted,
-                                                  virtual_src);
       break;
     }
+
+  virtual_src->watch =
+    meta_stage_watch_view (meta_stage,
+                           view,
+                           META_STAGE_WATCH_AFTER_PAINT,
+                           on_after_paint,
+                           virtual_src);
 }
 
 static void
@@ -264,22 +227,23 @@ on_monitors_changed (MetaMonitorManager             *monitor_manager,
   MetaScreenCastStreamSrc *src = META_SCREEN_CAST_STREAM_SRC (virtual_src);
   MetaStage *stage = META_STAGE (stage_from_src (src));
   MetaScreenCastStream *stream = meta_screen_cast_stream_src_get_stream (src);
+  ClutterStageView *view;
 
   meta_stage_remove_watch (stage, virtual_src->watch);
   virtual_src->watch = NULL;
-  add_watch (virtual_src);
+
+  view = view_from_src (src);
+  setup_view (virtual_src, view);
 
   meta_eis_viewport_notify_changed (META_EIS_VIEWPORT (stream));
 }
 
 static void
-init_record_callbacks (MetaScreenCastVirtualStreamSrc *virtual_src)
+setup_cursor_handling (MetaScreenCastVirtualStreamSrc *virtual_src)
 {
   MetaScreenCastStreamSrc *src = META_SCREEN_CAST_STREAM_SRC (virtual_src);
   MetaScreenCastStream *stream = meta_screen_cast_stream_src_get_stream (src);
   MetaBackend *backend = backend_from_src (src);
-  MetaMonitorManager *monitor_manager =
-    meta_backend_get_monitor_manager (backend);
   MetaCursorTracker *cursor_tracker =
     meta_backend_get_cursor_tracker (backend);
   ClutterStage *stage = stage_from_src (src);
@@ -287,6 +251,7 @@ init_record_callbacks (MetaScreenCastVirtualStreamSrc *virtual_src)
   switch (meta_screen_cast_stream_get_cursor_mode (stream))
     {
     case META_SCREEN_CAST_CURSOR_MODE_METADATA:
+      meta_cursor_tracker_track_position (cursor_tracker);
       virtual_src->position_invalidated_handler_id =
         g_signal_connect_after (cursor_tracker, "position-invalidated",
                                 G_CALLBACK (pointer_position_invalidated),
@@ -299,23 +264,13 @@ init_record_callbacks (MetaScreenCastVirtualStreamSrc *virtual_src)
         g_signal_connect_after (stage, "prepare-frame",
                                 G_CALLBACK (on_prepare_frame),
                                 virtual_src);
-      G_GNUC_FALLTHROUGH;
+      break;
     case META_SCREEN_CAST_CURSOR_MODE_EMBEDDED:
+      meta_cursor_tracker_track_position (cursor_tracker);
+      break;
     case META_SCREEN_CAST_CURSOR_MODE_HIDDEN:
-      add_watch (virtual_src);
       break;
     }
-
-  meta_screen_cast_stream_notify_is_configured (stream);
-
-  if (meta_screen_cast_stream_get_cursor_mode (stream) ==
-      META_SCREEN_CAST_CURSOR_MODE_EMBEDDED)
-    inhibit_hw_cursor (virtual_src);
-
-  virtual_src->monitors_changed_handler_id =
-    g_signal_connect (monitor_manager, "monitors-changed-internal",
-                      G_CALLBACK (on_monitors_changed),
-                      virtual_src);
 }
 
 static void
@@ -325,22 +280,25 @@ meta_screen_cast_virtual_stream_src_enable (MetaScreenCastStreamSrc *src)
     META_SCREEN_CAST_VIRTUAL_STREAM_SRC (src);
   MetaScreenCastStream *stream = meta_screen_cast_stream_src_get_stream (src);
   MetaBackend *backend = backend_from_src (src);
-  MetaCursorTracker *cursor_tracker = meta_backend_get_cursor_tracker (backend);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  ClutterStageView *view;
 
-  switch (meta_screen_cast_stream_get_cursor_mode (stream))
-    {
-    case META_SCREEN_CAST_CURSOR_MODE_METADATA:
-    case META_SCREEN_CAST_CURSOR_MODE_EMBEDDED:
-      meta_cursor_tracker_track_position (cursor_tracker);
-      break;
-    case META_SCREEN_CAST_CURSOR_MODE_HIDDEN:
-      break;
-    }
+  view = view_from_src (src);
+  if (view)
+    setup_view (virtual_src, view);
+
+  setup_cursor_handling (virtual_src);
+
+  meta_screen_cast_stream_notify_is_configured (stream);
+
+  virtual_src->monitors_changed_handler_id =
+    g_signal_connect (monitor_manager, "monitors-changed-internal",
+                      G_CALLBACK (on_monitors_changed),
+                      virtual_src);
 
-  init_record_callbacks (virtual_src);
   clutter_actor_queue_redraw_with_clip (CLUTTER_ACTOR (stage_from_src (src)),
                                         NULL);
-  clutter_stage_schedule_update (stage_from_src (src));
 }
 
 static void
@@ -355,9 +313,6 @@ meta_screen_cast_virtual_stream_src_disable (MetaScreenCastStreamSrc *src)
     meta_backend_get_monitor_manager (backend);
   ClutterStage *stage = stage_from_src (src);
 
-  if (virtual_src->hw_cursor_inhibited)
-    uninhibit_hw_cursor (virtual_src);
-
   if (virtual_src->watch)
     {
       meta_stage_remove_watch (META_STAGE (stage_from_src (src)),
@@ -738,22 +693,6 @@ meta_screen_cast_virtual_stream_src_new (MetaScreenCastVirtualStream  *virtual_s
                          NULL);
 }
 
-static gboolean
-meta_screen_cast_virtual_stream_src_is_cursor_inhibited (MetaHwCursorInhibitor *inhibitor)
-{
-  MetaScreenCastVirtualStreamSrc *virtual_src =
-    META_SCREEN_CAST_VIRTUAL_STREAM_SRC (inhibitor);
-
-  return is_cursor_in_stream (virtual_src);
-}
-
-static void
-hw_cursor_inhibitor_iface_init (MetaHwCursorInhibitorInterface *iface)
-{
-  iface->is_cursor_inhibited =
-    meta_screen_cast_virtual_stream_src_is_cursor_inhibited;
-}
-
 static void
 meta_screen_cast_virtual_stream_src_dispose (GObject *object)
 {
