@@ -79,21 +79,11 @@ meta_stage_impl_unrealize (ClutterStageWindow *stage_window)
 static gboolean
 meta_stage_impl_realize (ClutterStageWindow *stage_window)
 {
-  ClutterBackend *backend;
-
   meta_topic (META_DEBUG_BACKEND,
               "Realizing stage '%s' [%p]",
               G_OBJECT_TYPE_NAME (stage_window),
               stage_window);
 
-  backend = clutter_get_default_backend ();
-
-  if (backend->cogl_context == NULL)
-    {
-      g_warning ("Failed to realize stage: missing Cogl context");
-      return FALSE;
-    }
-
   return TRUE;
 }
 
@@ -157,7 +147,7 @@ paint_damage_region (ClutterStageWindow *stage_window,
   if (G_UNLIKELY (overlay_blue == NULL))
     {
       overlay_blue = cogl_pipeline_new (ctx);
-      cogl_color_init_from_4f (&blue_color, 0.0, 0.0, 0.2, 0.2);
+      cogl_color_init_from_4f (&blue_color, 0.0f, 0.0f, 0.2f, 0.2f);
       cogl_pipeline_set_color (overlay_blue, &blue_color);
     }
 
@@ -184,7 +174,7 @@ paint_damage_region (ClutterStageWindow *stage_window,
       if (G_UNLIKELY (overlay_red == NULL))
         {
           overlay_red = cogl_pipeline_new (ctx);
-          cogl_color_init_from_4f (&red_color, 0.2, 0.0, 0.0, 0.2);
+          cogl_color_init_from_4f (&red_color, 0.2f, 0.0f, 0.0f, 0.2f);
           cogl_pipeline_set_color (overlay_red, &red_color);
         }
 
@@ -243,11 +233,11 @@ queue_damage_region (ClutterStageWindow *stage_window,
 
       rect = mtk_region_get_rectangle (damage_region, i);
 
-      clutter_stage_view_transform_rect_to_onscreen (stage_view,
-                                                     &rect,
-                                                     fb_width,
-                                                     fb_height,
-                                                     &rect);
+      mtk_rectangle_transform (&rect,
+                               clutter_stage_view_get_transform (stage_view),
+                               fb_width,
+                               fb_height,
+                               &rect);
 
       damage[i * 4] = rect.x;
       /* y coordinate needs to be flipped for OpenGL */
@@ -338,6 +328,8 @@ swap_framebuffer (ClutterStageWindow *stage_window,
       meta_topic (META_DEBUG_BACKEND,
                   "fake offscreen swap (framebuffer: %p)",
                   framebuffer);
+
+      cogl_framebuffer_flush (framebuffer);
       meta_stage_view_perform_fake_swap (view, priv->global_frame_counter);
       priv->global_frame_counter++;
     }
@@ -450,11 +442,11 @@ transform_swap_region_to_onscreen (ClutterStageView *stage_view,
   for (i = 0; i < n_rects; i++)
     {
       rects[i] = mtk_region_get_rectangle (swap_region, i);
-      clutter_stage_view_transform_rect_to_onscreen (stage_view,
-                                                     &rects[i],
-                                                     width,
-                                                     height,
-                                                     &rects[i]);
+      mtk_rectangle_transform (&rects[i],
+                               clutter_stage_view_get_transform (stage_view),
+                               width,
+                               height,
+                               &rects[i]);
     }
   transformed_region = mtk_region_create_rectangles (rects, n_rects);
 
@@ -472,6 +464,7 @@ should_use_clipped_redraw (gboolean              is_full_redraw,
   gboolean can_blit_sub_buffer;
   gboolean can_use_clipped_redraw;
   gboolean is_warmed_up;
+  CoglContext *context = cogl_framebuffer_get_context (framebuffer);
 
   if (is_full_redraw)
     return FALSE;
@@ -490,7 +483,7 @@ should_use_clipped_redraw (gboolean              is_full_redraw,
     }
 
   can_blit_sub_buffer =
-    cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_SWAP_REGION);
+    cogl_context_has_winsys_feature (context, COGL_WINSYS_FEATURE_SWAP_REGION);
   can_use_clipped_redraw =
     _clutter_stage_window_can_clip_redraws (stage_window) &&
     (can_blit_sub_buffer || has_buffer_age);
@@ -509,6 +502,7 @@ meta_stage_impl_redraw_view_primary (MetaStageImpl    *stage_impl,
   ClutterStageWindow *stage_window = CLUTTER_STAGE_WINDOW (stage_impl);
   MetaStageView *view = META_STAGE_VIEW (stage_view);
   CoglFramebuffer *fb = clutter_stage_view_get_framebuffer (stage_view);
+  CoglContext *context = cogl_framebuffer_get_context (fb);
   CoglFramebuffer *onscreen = clutter_stage_view_get_onscreen (stage_view);
   MtkRectangle view_rect;
   gboolean is_full_redraw;
@@ -528,6 +522,9 @@ meta_stage_impl_redraw_view_primary (MetaStageImpl    *stage_impl,
 
   COGL_TRACE_BEGIN_SCOPED (RedrawViewPrimary,
                            "Meta::StageImpl::redraw_view_primary()");
+  COGL_TRACE_DEFINE_COUNTER_INT (RedrawViewPrimaryDamageArea,
+                                 "RedrawDamageArea",
+                                 "the damaged area of the redraw");
 
   clutter_stage_view_get_layout (stage_view, &view_rect);
   fb_scale = clutter_stage_view_get_scale (stage_view);
@@ -536,7 +533,7 @@ meta_stage_impl_redraw_view_primary (MetaStageImpl    *stage_impl,
 
   has_buffer_age =
     COGL_IS_ONSCREEN (onscreen) &&
-    cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_BUFFER_AGE);
+    cogl_context_has_winsys_feature (context, COGL_WINSYS_FEATURE_BUFFER_AGE);
 
   redraw_clip = clutter_stage_view_take_accumulated_redraw_clip (stage_view);
 
@@ -577,7 +574,7 @@ meta_stage_impl_redraw_view_primary (MetaStageImpl    *stage_impl,
         {
           queued_redraw_clip =
             scale_offset_and_clamp_region (fb_clip_region,
-                                           1.0 / fb_scale,
+                                           1.0f / fb_scale,
                                            view_rect.x,
                                            view_rect.y);
         }
@@ -658,7 +655,7 @@ meta_stage_impl_redraw_view_primary (MetaStageImpl    *stage_impl,
        */
       g_clear_pointer (&redraw_clip, mtk_region_unref);
       redraw_clip = scale_offset_and_clamp_region (fb_clip_region,
-                                                   1.0 / fb_scale,
+                                                   1.0f / fb_scale,
                                                    view_rect.x,
                                                    view_rect.y);
     }
@@ -687,6 +684,35 @@ meta_stage_impl_redraw_view_primary (MetaStageImpl    *stage_impl,
       paint_stage (stage_impl, stage_view, redraw_clip, frame);
     }
 
+#ifdef HAVE_PROFILER
+  if (G_UNLIKELY (cogl_is_tracing_enabled ()))
+    {
+      g_autoptr (GString) rects_str = NULL;
+      g_autofree char *area_str = NULL;
+      int n_rectangles, i;
+      int area = 0;
+
+      rects_str = g_string_new ("");
+      n_rectangles = mtk_region_num_rectangles (redraw_clip);
+
+      for (i = 0; i < n_rectangles; i++)
+        {
+          MtkRectangle rect = mtk_region_get_rectangle (redraw_clip, i);
+
+          area += mtk_rectangle_area (&rect);
+
+          g_string_append_printf (rects_str, " %d,%d,%d,%d",
+                                  rect.x, rect.y, rect.width, rect.height);
+        }
+
+      area_str = g_strdup_printf ("%d", area);
+      g_string_prepend (rects_str, area_str);
+
+      COGL_TRACE_DESCRIBE (RedrawViewPrimary, rects_str->str);
+      COGL_TRACE_SET_COUNTER_INT (RedrawViewPrimaryDamageArea, area);
+    }
+#endif
+
   g_clear_pointer (&redraw_clip, mtk_region_unref);
   g_clear_pointer (&fb_clip_region, mtk_region_unref);
 
