@@ -1696,7 +1696,16 @@ is_using_deadline_timer (MetaKmsImplDevice *impl_device)
       MetaKmsImpl *impl = meta_kms_impl_device_get_impl (impl_device);
       MetaThreadImpl *thread_impl = META_THREAD_IMPL (impl);
 
-      return meta_thread_impl_is_realtime (thread_impl);
+      switch (meta_thread_impl_get_scheduling_priority (thread_impl))
+        {
+        case META_SCHEDULING_PRIORITY_NORMAL:
+          return FALSE;
+        case META_SCHEDULING_PRIORITY_REALTIME:
+        case META_SCHEDULING_PRIORITY_HIGH_PRIORITY:
+          return TRUE;
+        }
+
+      g_assert_not_reached ();
     }
 }
 
