@@ -26,12 +26,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "backends/x11/meta-clutter-backend-x11.h"
 #include "compositor/compositor-private.h"
 #include "compositor/meta-module.h"
 #include "core/meta-inhibit-shortcuts-dialog-default-private.h"
 #include "core/window-private.h"
-#include "meta/meta-x11-errors.h"
 #include "meta/prefs.h"
 #include "meta/workspace.h"
 
@@ -115,7 +113,8 @@ on_prepare_shutdown (MetaContext       *context,
 }
 
 MetaPluginManager *
-meta_plugin_manager_new (MetaCompositor *compositor)
+meta_plugin_manager_new (MetaCompositor *compositor,
+                         GVariant       *plugin_options)
 {
   MetaBackend *backend = meta_compositor_get_backend (compositor);
   MetaMonitorManager *monitor_manager =
@@ -125,10 +124,21 @@ meta_plugin_manager_new (MetaCompositor *compositor)
   MetaDisplay *display;
   MetaContext *context;
 
+  if (plugin_options)
+    {
+      plugin = g_object_new (plugin_type,
+                             "options", plugin_options,
+                             NULL);
+    }
+  else
+    {
+      plugin = g_object_new (plugin_type, NULL);
+    }
+
   plugin_mgr = g_new0 (MetaPluginManager, 1);
   plugin_mgr->state = PLUGIN_MANAGER_STATE_STARTING;
   plugin_mgr->compositor = compositor;
-  plugin_mgr->plugin = plugin = g_object_new (plugin_type, NULL);
+  plugin_mgr->plugin = plugin;
 
   _meta_plugin_set_compositor (plugin, compositor);
 
@@ -333,6 +343,7 @@ meta_plugin_manager_filter_keybinding (MetaPluginManager *plugin_mgr,
   return FALSE;
 }
 
+#ifdef HAVE_X11
 gboolean
 meta_plugin_manager_xevent_filter (MetaPluginManager *plugin_mgr,
                                    XEvent            *xev)
@@ -341,6 +352,7 @@ meta_plugin_manager_xevent_filter (MetaPluginManager *plugin_mgr,
 
   return _meta_plugin_xevent_filter (plugin, xev);
 }
+#endif
 
 void
 meta_plugin_manager_confirm_display_change (MetaPluginManager *plugin_mgr)
