@@ -33,6 +33,17 @@
 #include "meta/util.h"
 #include "meta/window.h"
 
+enum
+{
+  PROP_0,
+
+  PROP_OPTIONS,
+
+  N_PROPS
+};
+
+static GParamSpec *obj_props[N_PROPS];
+
 typedef enum
 {
   ANIMATION_DESTROY,
@@ -68,6 +79,8 @@ struct _MetaTestShell
     ClutterGrab *grab;
     ClutterActor *prev_focus;
   } overview;
+
+  gboolean show_stage;
 };
 
 typedef struct _ActorPrivate
@@ -268,7 +281,6 @@ on_monitors_changed (MetaMonitorManager *monitor_manager,
       uint8_t red;
       uint8_t green;
       uint8_t blue;
-      ClutterColor color;
 
       meta_display_get_monitor_geometry (display, i, &rect);
 
@@ -282,10 +294,10 @@ on_monitors_changed (MetaMonitorManager *monitor_manager,
       blue = g_rand_int_range (rand, 0, 255);
       green = g_rand_int_range (rand, 0, 255);
       red = g_rand_int_range (rand, 0, 255);
-      clutter_color_init (&color, red, green, blue, 255);
 
       background = meta_background_new (display);
-      meta_background_set_color (background, &color);
+      meta_background_set_color (background,
+                                 &COGL_COLOR_INIT (red, green, blue, 255));
       meta_background_content_set_background (background_content, background);
       g_object_unref (background);
 
@@ -351,7 +363,8 @@ meta_test_shell_start (MetaPlugin *plugin)
                     G_CALLBACK (prepare_shutdown),
                     test_shell);
 
-  clutter_actor_show (meta_get_stage_for_display (display));
+  if (test_shell->show_stage)
+    clutter_actor_show (meta_get_stage_for_display (display));
 }
 
 static void
@@ -700,7 +713,7 @@ get_display_tile_preview (MetaDisplay *display)
       preview = g_new0 (DisplayTilePreview, 1);
 
       preview->actor = clutter_actor_new ();
-      clutter_actor_set_background_color (preview->actor, &CLUTTER_COLOR_INIT (0, 0, 255, 255));
+      clutter_actor_set_background_color (preview->actor, &COGL_COLOR_INIT (0, 0, 255, 255));
       clutter_actor_set_opacity (preview->actor, 100);
 
       clutter_actor_add_child (meta_get_window_group_for_display (display),
@@ -790,11 +803,46 @@ meta_test_shell_plugin_info (MetaPlugin *plugin)
   return &test_shell->info;
 }
 
+static void
+process_options (MetaTestShell *test_shell,
+                 GVariant      *options)
+{
+  gboolean show_stage;
+
+  if (!options)
+    return;
+
+  if (g_variant_lookup (options, "show-stage", "b", &show_stage))
+    test_shell->show_stage = show_stage;
+}
+
+static void
+meta_test_shell_set_property (GObject      *object,
+                              guint         prop_id,
+                              const GValue *value,
+                              GParamSpec   *pspec)
+{
+  MetaTestShell *test_shell = META_TEST_SHELL (object);
+
+  switch (prop_id)
+    {
+    case PROP_OPTIONS:
+      process_options (test_shell, g_value_get_variant (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
 static void
 meta_test_shell_class_init (MetaTestShellClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   MetaPluginClass *plugin_class  = META_PLUGIN_CLASS (klass);
 
+  object_class->set_property = meta_test_shell_set_property;
+
   plugin_class->start = meta_test_shell_start;
   plugin_class->map = meta_test_shell_map;
   plugin_class->minimize = meta_test_shell_minimize;
@@ -805,6 +853,15 @@ meta_test_shell_class_init (MetaTestShellClass *klass)
   plugin_class->kill_window_effects = meta_test_shell_kill_window_effects;
   plugin_class->kill_switch_workspace = meta_test_shell_kill_switch_workspace;
   plugin_class->plugin_info = meta_test_shell_plugin_info;
+
+  obj_props[PROP_OPTIONS] =
+    g_param_spec_variant ("options", NULL, NULL,
+                          G_VARIANT_TYPE_VARDICT,
+                          NULL,
+                          G_PARAM_CONSTRUCT_ONLY |
+                          G_PARAM_WRITABLE |
+                          G_PARAM_STATIC_STRINGS);
+  g_object_class_install_properties (object_class, N_PROPS, obj_props);
 }
 
 static void
@@ -815,4 +872,6 @@ meta_test_shell_init (MetaTestShell *test_shell)
   test_shell->info.author = "Mutter developers";
   test_shell->info.license = "GPL";
   test_shell->info.description = "This is test shell plugin implementation.";
+
+  test_shell->show_stage = TRUE;
 }
