@@ -12,12 +12,14 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
+/* Till https://gitlab.freedesktop.org/pipewire/pipewire/-/issues/4065 is fixed */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wfloat-conversion"
+
 #include "config.h"
 
 #include <pipewire/pipewire.h>
@@ -214,6 +216,10 @@ on_stream_state_changed (void                 *user_data,
 {
   Stream *stream = user_data;
 
+  g_debug ("New PipeWire stream (%u) state '%s'",
+           stream->pipewire_node_id,
+           pw_stream_state_as_string (state));
+
   switch (state)
     {
     case PW_STREAM_STATE_ERROR:
@@ -322,16 +328,17 @@ process_buffer (Stream            *stream,
 {
   process_buffer_metadata (stream, buffer);
 
-  if (buffer->datas[0].chunk->size == 0)
-    g_assert_not_reached ();
-  else if (buffer->datas[0].type == SPA_DATA_MemFd)
-    sanity_check_memfd (buffer);
-  else if (buffer->datas[0].type == SPA_DATA_DmaBuf)
-    g_assert_not_reached ();
-  else if (buffer->datas[0].type == SPA_DATA_MemPtr)
-    sanity_check_memptr (buffer);
-  else
-    g_assert_not_reached ();
+  if (buffer->datas[0].chunk->size != 0)
+    {
+      if (buffer->datas[0].type == SPA_DATA_MemFd)
+        sanity_check_memfd (buffer);
+      else if (buffer->datas[0].type == SPA_DATA_DmaBuf)
+        g_assert_not_reached ();
+      else if (buffer->datas[0].type == SPA_DATA_MemPtr)
+        sanity_check_memptr (buffer);
+      else
+        g_assert_not_reached ();
+    }
 }
 
 static void
@@ -341,14 +348,23 @@ on_stream_process (void *user_data)
   struct pw_buffer *next_buffer;
   struct pw_buffer *buffer = NULL;
 
+  if (!stream->pipewire_stream)
+    return;
+
   next_buffer = pw_stream_dequeue_buffer (stream->pipewire_stream);
+  if (next_buffer)
+    g_debug ("Dequeued buffer, queue previous");
   while (next_buffer)
     {
+
       buffer = next_buffer;
       next_buffer = pw_stream_dequeue_buffer (stream->pipewire_stream);
 
       if (next_buffer)
-        pw_stream_queue_buffer (stream->pipewire_stream, buffer);
+        {
+          g_debug ("Dequeued another buffer, queuing previous");
+          pw_stream_queue_buffer (stream->pipewire_stream, buffer);
+        }
     }
   if (!buffer)
     return;
@@ -433,18 +449,20 @@ stream_wait_for_cursor_position (Stream *stream,
     g_main_context_iteration (NULL, TRUE);
 }
 
-static G_GNUC_UNUSED void
+static void
 stream_wait_for_streaming (Stream *stream)
 {
+  g_debug ("Waiting for stream to stream");
   while (stream->state != PW_STREAM_STATE_STREAMING)
     g_main_context_iteration (NULL, TRUE);
 }
 
-static G_GNUC_UNUSED void
+static void
 stream_wait_for_render (Stream *stream)
 {
   int initial_buffer_count = stream->buffer_count;
 
+  g_debug ("Waiting for new buffer");
   while (stream->buffer_count == initial_buffer_count)
     g_main_context_iteration (NULL, TRUE);
 }
@@ -740,31 +758,37 @@ main (int    argc,
   Session *session;
   Stream *stream;
 
+  g_debug ("Initializing PipeWire");
   init_pipewire ();
 
+  g_debug ("Creating screen cast session");
   remote_desktop = remote_desktop_new ();
   screen_cast = screen_cast_new ();
   session = screen_cast_create_session (remote_desktop, screen_cast);
   stream = session_record_virtual (session, 50, 40);
 
+  g_debug ("Starting screen cast stream");
   session_start (session);
 
   /* Check that the display server handles events being emitted too early. */
   session_notify_absolute_pointer (session, stream, 2, 3);
 
   /* Check that we receive the initial frame */
-
+  g_debug ("Waiting for stream to be established");
   stream_wait_for_node (stream);
-  stream_wait_for_render (stream);
   stream_wait_for_streaming (stream);
+  stream_wait_for_render (stream);
   session_notify_absolute_pointer (session, stream, 6, 5);
   session_notify_absolute_pointer (session, stream, 5, 6);
+
+  g_debug ("Waiting for frame");
   stream_wait_for_render (stream);
   stream_wait_for_cursor_position (stream, 5, 6);
   g_assert_cmpint (stream->spa_format.size.width, ==, 50);
   g_assert_cmpint (stream->spa_format.size.height, ==, 40);
 
   /* Check that resizing works */
+  g_debug ("Resizing stream");
   stream_resize (stream, 70, 60);
   while (TRUE)
     {
@@ -781,6 +805,7 @@ main (int    argc,
   /* Check that resizing works */
   stream_resize (stream, 60, 60);
 
+  g_debug ("Stopping session");
   session_stop (session);
 
   stream_free (stream);
@@ -790,5 +815,9 @@ main (int    argc,
 
   release_pipewire ();
 
+  g_debug ("Done");
+
   return EXIT_SUCCESS;
 }
+
+#pragma GCC diagnostic pop
