@@ -415,6 +415,7 @@ xserver_died (GObject      *source,
         return;
 
       g_warning ("Failed to finish waiting for Xwayland: %s", error->message);
+      g_clear_error (&error);
     }
 
   x11_display_policy =
@@ -433,8 +434,6 @@ xserver_died (GObject      *source,
     }
   else if (x11_display_policy == META_X11_DISPLAY_POLICY_ON_DEMAND)
     {
-      g_autoptr (GError) error = NULL;
-
       if (display->x11_display)
         meta_display_shutdown_x11 (display);
 
@@ -1051,6 +1050,29 @@ meta_xwayland_shutdown (MetaWaylandCompositor *compositor)
     }
 }
 
+static void
+update_highest_monitor_scale (MetaXWaylandManager *manager)
+{
+  MetaWaylandCompositor *compositor = manager->compositor;
+  MetaContext *context = meta_wayland_compositor_get_context (compositor);
+  MetaBackend *backend = meta_context_get_backend (context);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  GList *logical_monitors;
+  GList *l;
+  double scale = 1.0;
+
+  logical_monitors = meta_monitor_manager_get_logical_monitors (monitor_manager);
+  for (l = logical_monitors; l; l = l->next)
+    {
+      MetaLogicalMonitor *logical_monitor = l->data;
+
+      scale = MAX (scale, meta_logical_monitor_get_scale (logical_monitor));
+    }
+
+  manager->highest_monitor_scale = scale;
+}
+
 gboolean
 meta_xwayland_init (MetaXWaylandManager    *manager,
                     MetaWaylandCompositor  *compositor,
@@ -1058,6 +1080,9 @@ meta_xwayland_init (MetaXWaylandManager    *manager,
                     GError                **error)
 {
   MetaContext *context = compositor->context;
+  MetaBackend *backend = meta_context_get_backend (context);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
   MetaX11DisplayPolicy policy;
   int display = 0;
 
@@ -1115,12 +1140,16 @@ meta_xwayland_init (MetaXWaylandManager    *manager,
 
   if (policy != META_X11_DISPLAY_POLICY_DISABLED)
     manager->prepare_shutdown_id = g_signal_connect (compositor, "prepare-shutdown",
-                                                     G_CALLBACK (meta_xwayland_shutdown), 
+                                                     G_CALLBACK (meta_xwayland_shutdown),
                                                      NULL);
 
   /* Xwayland specific protocol, needs to be filtered out for all other clients */
   meta_xwayland_grab_keyboard_init (compositor);
 
+  g_signal_connect_swapped (monitor_manager, "monitors-changed-internal",
+                            G_CALLBACK (update_highest_monitor_scale), manager);
+  update_highest_monitor_scale (manager);
+
   return TRUE;
 }
 
@@ -1180,6 +1209,7 @@ on_x11_display_setup (MetaDisplay         *display,
 {
   MetaX11Display *x11_display = meta_display_get_x11_display (display);
 
+  meta_x11_display_redirect_windows (x11_display, display);
   meta_xwayland_init_dnd (x11_display);
   meta_xwayland_init_xrandr (manager, x11_display);
 }
@@ -1216,15 +1246,19 @@ meta_xwayland_set_primary_output (MetaX11Display *x11_display)
   MetaMonitorManager *monitor_manager =
     monitor_manager_from_x11_display (x11_display);
   XRRScreenResources *resources;
-  MetaLogicalMonitor *primary_monitor;
+  MetaLogicalMonitor *primary_logical_monitor;
+  GList *monitors;
+  MetaMonitor *primary_monitor;
   int i;
 
-  primary_monitor =
+  primary_logical_monitor =
     meta_monitor_manager_get_primary_logical_monitor (monitor_manager);
-
-  if (!primary_monitor)
+  if (!primary_logical_monitor)
     return;
 
+  monitors = meta_logical_monitor_get_monitors (primary_logical_monitor);
+  primary_monitor = g_list_first (monitors)->data;
+
   resources = XRRGetScreenResourcesCurrent (xdisplay,
                                             DefaultRootWindow (xdisplay));
   if (!resources)
@@ -1235,34 +1269,21 @@ meta_xwayland_set_primary_output (MetaX11Display *x11_display)
     {
       RROutput output_id = resources->outputs[i];
       XRROutputInfo *xrandr_output;
-      XRRCrtcInfo *crtc_info = NULL;
-      MtkRectangle crtc_geometry;
 
       xrandr_output = XRRGetOutputInfo (xdisplay, resources, output_id);
       if (!xrandr_output)
         continue;
 
-      if (xrandr_output->crtc)
-        crtc_info = XRRGetCrtcInfo (xdisplay, resources, xrandr_output->crtc);
-
-      XRRFreeOutputInfo (xrandr_output);
-
-      if (!crtc_info)
-        continue;
-
-      crtc_geometry.x = crtc_info->x;
-      crtc_geometry.y = crtc_info->y;
-      crtc_geometry.width = crtc_info->width;
-      crtc_geometry.height = crtc_info->height;
-
-      XRRFreeCrtcInfo (crtc_info);
-
-      if (mtk_rectangle_equal (&crtc_geometry, &primary_monitor->rect))
+      if (g_strcmp0 (xrandr_output->name,
+                     meta_monitor_get_connector (primary_monitor)) == 0)
         {
           XRRSetOutputPrimary (xdisplay, DefaultRootWindow (xdisplay),
                                output_id);
+          XRRFreeOutputInfo (xrandr_output);
           break;
         }
+
+      XRRFreeOutputInfo (xrandr_output);
     }
   mtk_x11_error_trap_pop (x11_display->xdisplay);
 
@@ -1312,3 +1333,50 @@ meta_xwayland_set_should_enable_ei_portal (MetaXWaylandManager  *manager,
 {
   manager->should_enable_ei_portal = should_enable_ei_portal;
 }
+
+int
+meta_xwayland_get_effective_scale (MetaXWaylandManager *manager)
+{
+  MetaWaylandCompositor *compositor = manager->compositor;
+  MetaContext *context = meta_wayland_compositor_get_context (compositor);
+  MetaBackend *backend = meta_context_get_backend (context);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  MetaSettings *settings = meta_backend_get_settings (backend);
+
+  switch (meta_monitor_manager_get_layout_mode (monitor_manager))
+    {
+    case META_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
+      break;
+
+    case META_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
+      if (meta_settings_is_experimental_feature_enabled (settings,
+                                                         META_EXPERIMENTAL_FEATURE_XWAYLAND_NATIVE_SCALING) &&
+          meta_settings_is_experimental_feature_enabled (settings,
+                                                         META_EXPERIMENTAL_FEATURE_SCALE_MONITOR_FRAMEBUFFER))
+        return (int) ceil (manager->highest_monitor_scale);
+    }
+
+  return 1;
+}
+
+int
+meta_xwayland_get_x11_ui_scaling_factor (MetaXWaylandManager *manager)
+{
+  MetaWaylandCompositor *compositor = manager->compositor;
+  MetaContext *context = meta_wayland_compositor_get_context (compositor);
+  MetaBackend *backend = meta_context_get_backend (context);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  MetaSettings *settings = meta_backend_get_settings (backend);
+
+  switch (meta_monitor_manager_get_layout_mode (monitor_manager))
+    {
+    case META_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
+      return meta_settings_get_ui_scaling_factor (settings);
+    case META_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
+      return meta_xwayland_get_effective_scale (manager);
+    }
+
+  g_assert_not_reached ();
+}
