@@ -44,12 +44,12 @@
 #include <string.h>
 
 #include "compositor/compositor-private.h"
-#include "core/frame.h"
 #include "core/meta-workspace-manager-private.h"
 #include "core/util-private.h"
-#include "meta/group.h"
+#include "meta/meta-x11-group.h"
 #include "mtk/mtk-x11.h"
 #include "x11/meta-x11-display-private.h"
+#include "x11/meta-x11-frame.h"
 #include "x11/window-x11-private.h"
 #include "x11/window-x11.h"
 #include "x11/xprops.h"
@@ -305,10 +305,11 @@ reload_icon_geometry (MetaWindow    *window,
         {
           MtkRectangle geometry;
 
-          geometry.x = (int)value->v.cardinal_list.cardinals[0];
-          geometry.y = (int)value->v.cardinal_list.cardinals[1];
-          geometry.width = (int)value->v.cardinal_list.cardinals[2];
-          geometry.height = (int)value->v.cardinal_list.cardinals[3];
+          geometry = MTK_RECTANGLE_INIT (value->v.cardinal_list.cardinals[0],
+                                         value->v.cardinal_list.cardinals[1],
+                                         value->v.cardinal_list.cardinals[2],
+                                         value->v.cardinal_list.cardinals[3]);
+          meta_window_protocol_to_stage_rect (window, &geometry, &geometry);
 
           meta_window_set_icon_geometry (window, &geometry);
         }
@@ -324,13 +325,16 @@ meta_window_set_custom_frame_extents (MetaWindow      *window,
                                       MetaFrameBorder *extents,
                                       gboolean         is_initial)
 {
+  MetaWindowX11 *window_x11 = META_WINDOW_X11 (window);
+  MetaWindowX11Private *priv =
+    meta_window_x11_get_private (window_x11);
   if (extents)
     {
-      if (window->has_custom_frame_extents &&
+      if (priv->has_custom_frame_extents &&
           memcmp (&window->custom_frame_extents, extents, sizeof (MetaFrameBorder)) == 0)
         return;
 
-      window->has_custom_frame_extents = TRUE;
+      priv->has_custom_frame_extents = TRUE;
       window->custom_frame_extents = *extents;
 
       /* If we're setting the frame extents on map, then this is telling
@@ -346,10 +350,10 @@ meta_window_set_custom_frame_extents (MetaWindow      *window,
     }
   else
     {
-      if (!window->has_custom_frame_extents)
+      if (!priv->has_custom_frame_extents)
         return;
 
-      window->has_custom_frame_extents = FALSE;
+      priv->has_custom_frame_extents = FALSE;
       memset (&window->custom_frame_extents, 0, sizeof (window->custom_frame_extents));
     }
 
@@ -370,11 +374,27 @@ reload_gtk_frame_extents (MetaWindow    *window,
         }
       else
         {
+          int left, right, top, bottom;
           MetaFrameBorder extents;
-          extents.left   = (int)value->v.cardinal_list.cardinals[0];
-          extents.right  = (int)value->v.cardinal_list.cardinals[1];
-          extents.top    = (int)value->v.cardinal_list.cardinals[2];
-          extents.bottom = (int)value->v.cardinal_list.cardinals[3];
+
+          meta_window_protocol_to_stage_point (window,
+                                               value->v.cardinal_list.cardinals[0],
+                                               value->v.cardinal_list.cardinals[1],
+                                               &left,
+                                               &right,
+                                               MTK_ROUNDING_STRATEGY_GROW);
+          meta_window_protocol_to_stage_point (window,
+                                               value->v.cardinal_list.cardinals[2],
+                                               value->v.cardinal_list.cardinals[3],
+                                               &top,
+                                               &bottom,
+                                               MTK_ROUNDING_STRATEGY_GROW);
+
+          extents.left = left;
+          extents.right = right;
+          extents.top = top;
+          extents.bottom = bottom;
+
           meta_window_set_custom_frame_extents (window, &extents, initial);
         }
     }
@@ -649,6 +669,7 @@ reload_opaque_region (MetaWindow    *window,
                       gboolean       initial)
 {
   MtkRegion *opaque_region = NULL;
+  MetaFrame *frame;
 
   if (value->type != META_PROP_VALUE_INVALID)
     {
@@ -676,13 +697,15 @@ reload_opaque_region (MetaWindow    *window,
       i = 0;
       while (i < nitems)
         {
+          MtkRectangle region_rect = MTK_RECTANGLE_INIT (region[i + 0],
+                                                         region[i + 1],
+                                                         region[i + 2],
+                                                         region[i + 3]);
           MtkRectangle *rect = &rects[rect_index];
 
-          rect->x = region[i++];
-          rect->y = region[i++];
-          rect->width = region[i++];
-          rect->height = region[i++];
+          meta_window_protocol_to_stage_rect (window, &region_rect, rect);
 
+          i += 4;
           rect_index++;
         }
 
@@ -692,10 +715,11 @@ reload_opaque_region (MetaWindow    *window,
     }
 
  out:
+  frame = meta_window_x11_get_frame (window);
   if (value->source_xwindow == meta_window_x11_get_xwindow (window))
     meta_window_set_opaque_region (window, opaque_region);
-  else if (window->frame && value->source_xwindow == window->frame->xwindow)
-    meta_frame_set_opaque_region (window->frame, opaque_region);
+  else if (frame && value->source_xwindow == frame->xwindow)
+    meta_frame_set_opaque_region (frame, opaque_region);
 
   g_clear_pointer (&opaque_region, mtk_region_unref);
 }
@@ -1037,11 +1061,12 @@ reload_update_counter (MetaWindow    *window,
   if (value->type != META_PROP_VALUE_INVALID)
     {
       MetaSyncCounter *sync_counter;
+      MetaFrame *frame = meta_window_x11_get_frame (window);
 
       if (value->source_xwindow == meta_window_x11_get_xwindow (window))
         sync_counter = meta_window_x11_get_sync_counter (window);
-      else if (window->frame && value->source_xwindow == window->frame->xwindow)
-        sync_counter = meta_frame_get_sync_counter (window->frame);
+      else if (frame && value->source_xwindow == frame->xwindow)
+        sync_counter = meta_frame_get_sync_counter (frame);
       else
         g_assert_not_reached ();
 
@@ -1213,346 +1238,48 @@ hints_have_changed (const MetaSizeHints *old,
          FLAG_TOGGLED_OFF (old, new, META_SIZE_HINTS_PROGRAM_WIN_GRAVITY);
 }
 
-void
-meta_set_normal_hints (MetaWindow *window,
-                       XSizeHints *hints)
+static void
+scale_size_hints (MetaWindow    *window,
+                  MetaSizeHints *hints)
 {
-  int x, y, w, h;
-  double minr, maxr;
-  /* Some convenience vars */
-  int minw, minh, maxw, maxh;   /* min/max width/height                      */
-  int basew, baseh, winc, hinc; /* base width/height, width/height increment */
+  meta_window_protocol_to_stage_point (window,
+                                       hints->x, hints->y,
+                                       &hints->x, &hints->y,
+                                       MTK_ROUNDING_STRATEGY_SHRINK);
+  meta_window_protocol_to_stage_point (window,
+                                       hints->width, hints->height,
+                                       &hints->width, &hints->height,
+                                       MTK_ROUNDING_STRATEGY_GROW);
 
-  /* Save the last ConfigureRequest, which we put here.
-   * Values here set in the hints are supposed to
-   * be ignored.
-   */
-  x = window->size_hints.x;
-  y = window->size_hints.y;
-  w = window->size_hints.width;
-  h = window->size_hints.height;
+  meta_window_protocol_to_stage_point (window,
+                                       hints->min_width, hints->min_height,
+                                       &hints->min_width, &hints->min_height,
+                                       MTK_ROUNDING_STRATEGY_GROW);
 
-  /* as far as I can tell, value->v.size_hints.flags is just to
-   * check whether we had old-style normal hints without gravity,
-   * base size as returned by XGetNormalHints(), so we don't
-   * really use it as we fixup window->size_hints to have those
-   * fields if they're missing.
-   */
+  meta_window_protocol_to_stage_point (window,
+                                       hints->max_width, hints->max_height,
+                                       &hints->max_width, &hints->max_height,
+                                       MTK_ROUNDING_STRATEGY_GROW);
 
-  /*
-   * When the window is first created, NULL hints will
-   * be passed in which will initialize all of the fields
-   * as if flags were zero
-   */
-  if (hints)
-    window->size_hints = *(MetaSizeHints*)(hints);
-  else
-    window->size_hints.flags = 0;
+  meta_window_protocol_to_stage_point (window,
+                                       hints->width_inc, hints->height_inc,
+                                       &hints->width_inc, &hints->height_inc,
+                                       MTK_ROUNDING_STRATEGY_ROUND);
 
-  /* Put back saved ConfigureRequest. */
-  window->size_hints.x = x;
-  window->size_hints.y = y;
-  window->size_hints.width = w;
-  window->size_hints.height = h;
+  meta_window_protocol_to_stage_point (window,
+                                       hints->min_aspect.x, hints->min_aspect.y,
+                                       &hints->min_aspect.x, &hints->min_aspect.y,
+                                       MTK_ROUNDING_STRATEGY_ROUND);
 
-  /* Get base size hints */
-  if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_BASE_SIZE)
-    {
-      meta_topic (META_DEBUG_GEOMETRY, "Window %s sets base size %d x %d",
-                  window->desc,
-                  window->size_hints.base_width,
-                  window->size_hints.base_height);
-    }
-  else if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_MIN_SIZE)
-    {
-      window->size_hints.base_width = window->size_hints.min_width;
-      window->size_hints.base_height = window->size_hints.min_height;
-    }
-  else
-    {
-      window->size_hints.base_width = 0;
-      window->size_hints.base_height = 0;
-    }
-  window->size_hints.flags |= META_SIZE_HINTS_PROGRAM_BASE_SIZE;
+  meta_window_protocol_to_stage_point (window,
+                                       hints->max_aspect.x, hints->max_aspect.y,
+                                       &hints->max_aspect.x, &hints->max_aspect.y,
+                                       MTK_ROUNDING_STRATEGY_ROUND);
 
-  /* Get min size hints */
-  if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_MIN_SIZE)
-    {
-      meta_topic (META_DEBUG_GEOMETRY, "Window %s sets min size %d x %d",
-                  window->desc,
-                  window->size_hints.min_width,
-                  window->size_hints.min_height);
-    }
-  else if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_BASE_SIZE)
-    {
-      window->size_hints.min_width = window->size_hints.base_width;
-      window->size_hints.min_height = window->size_hints.base_height;
-    }
-  else
-    {
-      window->size_hints.min_width = 0;
-      window->size_hints.min_height = 0;
-    }
-  window->size_hints.flags |= META_SIZE_HINTS_PROGRAM_MIN_SIZE;
-
-  /* Get max size hints */
-  if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_MAX_SIZE)
-    {
-      meta_topic (META_DEBUG_GEOMETRY, "Window %s sets max size %d x %d",
-                  window->desc,
-                  window->size_hints.max_width,
-                  window->size_hints.max_height);
-    }
-  else
-    {
-      window->size_hints.max_width = G_MAXINT;
-      window->size_hints.max_height = G_MAXINT;
-      window->size_hints.flags |= META_SIZE_HINTS_PROGRAM_MAX_SIZE;
-    }
-
-  /* Get resize increment hints */
-  if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_RESIZE_INCREMENTS)
-    {
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets resize width inc: %d height inc: %d",
-                  window->desc,
-                  window->size_hints.width_inc,
-                  window->size_hints.height_inc);
-    }
-  else
-    {
-      window->size_hints.width_inc = 1;
-      window->size_hints.height_inc = 1;
-      window->size_hints.flags |= META_SIZE_HINTS_PROGRAM_RESIZE_INCREMENTS;
-    }
-
-  /* Get aspect ratio hints */
-  if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_ASPECT)
-    {
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets min_aspect: %d/%d max_aspect: %d/%d",
-                  window->desc,
-                  window->size_hints.min_aspect.x,
-                  window->size_hints.min_aspect.y,
-                  window->size_hints.max_aspect.x,
-                  window->size_hints.max_aspect.y);
-    }
-  else
-    {
-      window->size_hints.min_aspect.x = 1;
-      window->size_hints.min_aspect.y = G_MAXINT;
-      window->size_hints.max_aspect.x = G_MAXINT;
-      window->size_hints.max_aspect.y = 1;
-      window->size_hints.flags |= META_SIZE_HINTS_PROGRAM_ASPECT;
-    }
-
-  /* Get gravity hint */
-  if (window->size_hints.flags & META_SIZE_HINTS_PROGRAM_WIN_GRAVITY)
-    {
-      meta_topic (META_DEBUG_GEOMETRY, "Window %s sets gravity %d",
-                  window->desc,
-                  window->size_hints.win_gravity);
-    }
-  else
-    {
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s doesn't set gravity, using NW",
-                  window->desc);
-      window->size_hints.win_gravity = META_GRAVITY_NORTH_WEST;
-      window->size_hints.flags |= META_SIZE_HINTS_PROGRAM_WIN_GRAVITY;
-    }
-
-  /*** Lots of sanity checking ***/
-
-  /* Verify all min & max hints are at least 1 pixel */
-  if (window->size_hints.min_width < 1)
-    {
-      /* someone is on crack */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets min width to 0, which makes no sense",
-                  window->desc);
-      window->size_hints.min_width = 1;
-    }
-  if (window->size_hints.max_width < 1)
-    {
-      /* another cracksmoker */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets max width to 0, which makes no sense",
-                  window->desc);
-      window->size_hints.max_width = 1;
-    }
-  if (window->size_hints.min_height < 1)
-    {
-      /* another cracksmoker */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets min height to 0, which makes no sense",
-                  window->desc);
-      window->size_hints.min_height = 1;
-    }
-  if (window->size_hints.max_height < 1)
-    {
-      /* another cracksmoker */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets max height to 0, which makes no sense",
-                  window->desc);
-      window->size_hints.max_height = 1;
-    }
-
-  /* Verify size increment hints are at least 1 pixel */
-  if (window->size_hints.width_inc < 1)
-    {
-      /* app authors find so many ways to smoke crack */
-      window->size_hints.width_inc = 1;
-      meta_topic (META_DEBUG_GEOMETRY, "Corrected 0 width_inc to 1");
-    }
-  if (window->size_hints.height_inc < 1)
-    {
-      /* another cracksmoker */
-      window->size_hints.height_inc = 1;
-      meta_topic (META_DEBUG_GEOMETRY, "Corrected 0 height_inc to 1");
-    }
-  /* divide by 0 cracksmokers; note that x & y in (min|max)_aspect are
-   * numerator & denominator
-   */
-  if (window->size_hints.min_aspect.y < 1)
-    window->size_hints.min_aspect.y = 1;
-  if (window->size_hints.max_aspect.y < 1)
-    window->size_hints.max_aspect.y = 1;
-
-  minw  = window->size_hints.min_width;  minh  = window->size_hints.min_height;
-  maxw  = window->size_hints.max_width;  maxh  = window->size_hints.max_height;
-  basew = window->size_hints.base_width; baseh = window->size_hints.base_height;
-  winc  = window->size_hints.width_inc;  hinc  = window->size_hints.height_inc;
-
-  /* Make sure min and max size hints are consistent with the base + increment
-   * size hints.  If they're not, it's not a real big deal, but it means the
-   * effective min and max size are more restrictive than the application
-   * specified values.
-   */
-  if ((minw - basew) % winc != 0)
-    {
-      /* Take advantage of integer division throwing away the remainder... */
-      window->size_hints.min_width = basew + ((minw - basew)/winc + 1)*winc;
-
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s has width_inc (%d) that does not evenly divide "
-                  "min_width - base_width (%d - %d); thus effective "
-                  "min_width is really %d",
-                  window->desc,
-                  winc, minw, basew, window->size_hints.min_width);
-      minw = window->size_hints.min_width;
-    }
-  if (maxw != G_MAXINT && (maxw - basew) % winc != 0)
-    {
-      /* Take advantage of integer division throwing away the remainder... */
-      window->size_hints.max_width = basew + ((maxw - basew)/winc)*winc;
-
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s has width_inc (%d) that does not evenly divide "
-                  "max_width - base_width (%d - %d); thus effective "
-                  "max_width is really %d",
-                  window->desc,
-                  winc, maxw, basew, window->size_hints.max_width);
-      maxw = window->size_hints.max_width;
-    }
-  if ((minh - baseh) % hinc != 0)
-    {
-      /* Take advantage of integer division throwing away the remainder... */
-      window->size_hints.min_height = baseh + ((minh - baseh)/hinc + 1)*hinc;
-
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s has height_inc (%d) that does not evenly divide "
-                  "min_height - base_height (%d - %d); thus effective "
-                  "min_height is really %d",
-                  window->desc,
-                  hinc, minh, baseh, window->size_hints.min_height);
-      minh = window->size_hints.min_height;
-    }
-  if (maxh != G_MAXINT && (maxh - baseh) % hinc != 0)
-    {
-      /* Take advantage of integer division throwing away the remainder... */
-      window->size_hints.max_height = baseh + ((maxh - baseh)/hinc)*hinc;
-
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s has height_inc (%d) that does not evenly divide "
-                  "max_height - base_height (%d - %d); thus effective "
-                  "max_height is really %d",
-                  window->desc,
-                  hinc, maxh, baseh, window->size_hints.max_height);
-      maxh = window->size_hints.max_height;
-    }
-
-  /* make sure maximum size hints are compatible with minimum size hints; min
-   * size hints take precedence.
-   */
-  if (window->size_hints.max_width < window->size_hints.min_width)
-    {
-      /* another cracksmoker */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets max width %d less than min width %d, "
-                  "disabling resize",
-                  window->desc,
-                  window->size_hints.max_width,
-                  window->size_hints.min_width);
-      maxw = window->size_hints.max_width = window->size_hints.min_width;
-    }
-  if (window->size_hints.max_height < window->size_hints.min_height)
-    {
-      /* another cracksmoker */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets max height %d less than min height %d, "
-                  "disabling resize",
-                  window->desc,
-                  window->size_hints.max_height,
-                  window->size_hints.min_height);
-      maxh = window->size_hints.max_height = window->size_hints.min_height;
-    }
-
-  /* Make sure the aspect ratio hints are sane. */
-  minr =         window->size_hints.min_aspect.x /
-         (double)window->size_hints.min_aspect.y;
-  maxr =         window->size_hints.max_aspect.x /
-         (double)window->size_hints.max_aspect.y;
-  if (minr > maxr)
-    {
-      /* another cracksmoker; not even minimally (self) consistent */
-      meta_topic (META_DEBUG_GEOMETRY,
-                  "Window %s sets min aspect ratio larger than max aspect "
-                  "ratio; disabling aspect ratio constraints.",
-                  window->desc);
-      window->size_hints.min_aspect.x = 1;
-      window->size_hints.min_aspect.y = G_MAXINT;
-      window->size_hints.max_aspect.x = G_MAXINT;
-      window->size_hints.max_aspect.y = 1;
-    }
-  else /* check consistency of aspect ratio hints with other hints */
-    {
-      if (minh > 0 && minr > (maxw / (double)minh))
-        {
-          /* another cracksmoker */
-          meta_topic (META_DEBUG_GEOMETRY,
-                      "Window %s sets min aspect ratio larger than largest "
-                      "aspect ratio possible given min/max size constraints; "
-                      "disabling min aspect ratio constraint.",
-                      window->desc);
-          window->size_hints.min_aspect.x = 1;
-          window->size_hints.min_aspect.y = G_MAXINT;
-        }
-      if (maxr < (minw / (double)maxh))
-        {
-          /* another cracksmoker */
-          meta_topic (META_DEBUG_GEOMETRY,
-                      "Window %s sets max aspect ratio smaller than smallest "
-                      "aspect ratio possible given min/max size constraints; "
-                      "disabling max aspect ratio constraint.",
-                      window->desc);
-          window->size_hints.max_aspect.x = G_MAXINT;
-          window->size_hints.max_aspect.y = 1;
-        }
-      /* FIXME: Would be nice to check that aspect ratios are
-       * consistent with base and size increment constraints.
-       */
-    }
+  meta_window_protocol_to_stage_point (window,
+                                       hints->base_width, hints->base_height,
+                                       &hints->base_width, &hints->base_height,
+                                       MTK_ROUNDING_STRATEGY_GROW);
 }
 
 static void
@@ -1569,7 +1296,18 @@ reload_normal_hints (MetaWindow    *window,
 
       old_hints = window->size_hints;
 
-      meta_set_normal_hints (window, value->v.size_hints.hints);
+      if (value->v.size_hints.hints)
+        {
+          MetaSizeHints new_hints;
+
+          new_hints = *(MetaSizeHints *) value->v.size_hints.hints;
+          scale_size_hints (window, &new_hints);
+          meta_window_set_normal_hints (window, &new_hints);
+        }
+      else
+        {
+          meta_window_set_normal_hints (window, NULL);
+        }
 
       hints_have_differences = hints_have_changed (&old_hints,
                                                    &window->size_hints);
